@@ -37,8 +37,8 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 #endif
 
 #define VERSION_NUMBER 1
-#define RELEASE_NUMBER 0
-#define UPDATE_NUMBER 15
+#define RELEASE_NUMBER 1
+#define UPDATE_NUMBER 02
 
 #define MAXLINE 5000 
 
@@ -98,7 +98,7 @@ typedef uint32_t ArrayIdx;
 
 // Atomic word
 typedef struct kmer_st Kmer;
-typedef int64_t KmerKey;
+typedef uint64_t KmerKey;
 
 // Just a sequence string, but with just two bits per character
 typedef struct tString_st TightString;
