@@ -18,10 +18,16 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <limits.h>
+#include <sys/time.h>
+
+#ifdef OPENMP
+#include <omp.h>
+#endif
 
 #include "globals.h"
 #include "graph.h"
@@ -32,12 +38,102 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 #include "utility.h"
 #include "kmer.h"
 #include "kmerOccurenceTable.h"
+#include "roadMap.h"
 
 #define ADENINE 0
 #define CYTOSINE 1
 #define GUANINE 2
 #define THYMINE 3
 
+
+//////////////////////////////////////////////////////////
+// Node Locking
+//////////////////////////////////////////////////////////
+
+#ifdef OPENMP
+
+/* Array of per-node locks */
+
+static omp_lock_t *nodeLocks = NULL;
+
+static void
+createNodeLocks(Graph *graph)
+{
+	IDnum nbNodes;
+	IDnum nodeIndex;
+
+	nbNodes = nodeCount(graph) + 1;
+	if (nodeLocks)
+		free (nodeLocks);
+	nodeLocks = mallocOrExit(nbNodes, omp_lock_t);
+
+	#pragma omp parallel for
+	for (nodeIndex = 0; nodeIndex < nbNodes; nodeIndex++)
+		omp_init_lock(nodeLocks + nodeIndex);
+}
+
+static inline void lockNode(Node *node)
+{
+	IDnum nodeID = getNodeID(node);
+
+	if (nodeID < 0)
+		nodeID = -nodeID;
+	omp_set_lock (nodeLocks + nodeID);
+}
+
+/* Assumes node is already locked */
+static inline void lockTwoNodes(Node *node, Node *node2)
+{
+	IDnum nodeID = getNodeID(node);
+	IDnum node2ID = getNodeID(node2);
+
+	if (nodeID < 0)
+		nodeID = -nodeID;
+	if (node2ID < 0)
+		node2ID = -node2ID;
+
+	if (nodeID == node2ID)
+		return;
+
+	/* Lock lowest ID first to avoid deadlocks */
+	if (nodeID < node2ID)
+	{
+		omp_set_lock (nodeLocks + node2ID);
+	}
+	else if (!omp_test_lock (nodeLocks + node2ID))
+	{
+		omp_unset_lock (nodeLocks + nodeID);
+		omp_set_lock (nodeLocks + node2ID);
+		omp_set_lock (nodeLocks + nodeID);
+	}
+}
+
+static inline void unLockTwoNodes(Node *node, Node *node2)
+{
+	IDnum nodeID = getNodeID(node);
+	IDnum node2ID = getNodeID(node2);
+
+	if (nodeID < 0)
+		nodeID = -nodeID;
+	if (node2ID < 0)
+		node2ID = -node2ID;
+
+	omp_unset_lock (nodeLocks + nodeID);
+	if (nodeID != node2ID)
+		omp_unset_lock (nodeLocks + node2ID);
+}
+
+static inline void unLockNode(Node *node)
+{
+	IDnum nodeID = getNodeID(node);
+
+	if (nodeID < 0)
+		nodeID = -nodeID;
+	omp_unset_lock (nodeLocks + nodeID);
+}
+
+#endif
+
 //////////////////////////////////////////////////////////
 // Node Lists
 //////////////////////////////////////////////////////////
@@ -49,40 +145,104 @@ struct smallNodeList_st {
 } ATTRIBUTE_PACKED;
 
 static RecycleBin *smallNodeListMemory = NULL;
-static SmallNodeList *nodePile = NULL;
 
 #define BLOCKSIZE 1000
 
+#ifdef OPENMP
+static void initSmallNodeListMemory(void)
+{
+	int n = omp_get_max_threads();
+
+#pragma omp critical
+	{
+		if (smallNodeListMemory == NULL)
+			smallNodeListMemory = newRecycleBinArray(n, sizeof(SmallNodeList), BLOCKSIZE);
+	}
+}
+#endif
+
 static SmallNodeList *allocateSmallNodeList()
 {
+#ifdef OPENMP
+#ifdef DEBUG
 	if (smallNodeListMemory == NULL)
-		smallNodeListMemory =
-		    newRecycleBin(sizeof(SmallNodeList), BLOCKSIZE);
+	{
+		velvetLog("The memory for small nodes seems uninitialised, "
+				"this is probably a bug, aborting.\n");
+		abort();
+	}
+#endif
+	return allocatePointer(getRecycleBinInArray(smallNodeListMemory,
+				omp_get_thread_num()));
+#else
+	if (smallNodeListMemory == NULL)
+		smallNodeListMemory = newRecycleBin(sizeof(SmallNodeList), BLOCKSIZE);
 
 	return allocatePointer(smallNodeListMemory);
+#endif
 }
 
 static void deallocateSmallNodeList(SmallNodeList * smallNodeList)
 {
+#ifdef OPENMP
+	deallocatePointer(getRecycleBinInArray(smallNodeListMemory,
+					       omp_get_thread_num()),
+			  smallNodeList);
+#else
 	deallocatePointer(smallNodeListMemory, smallNodeList);
+#endif
 }
 
-static void memorizeNode(Node * node)
+static void destroySmallNodeListMemmory(void)
+{
+	if (smallNodeListMemory != NULL)
+	{
+#ifdef OPENMP
+		destroyRecycleBinArray(smallNodeListMemory);
+#else
+		destroyRecycleBin(smallNodeListMemory);
+#endif
+		smallNodeListMemory = NULL;
+	}
+}
+
+static inline void memorizeNode(Node * node, SmallNodeList ** nodePile)
 {
 	SmallNodeList *list = allocateSmallNodeList();
 	list->node = node;
-	list->next = nodePile;
-	nodePile = list;
+	list->next = *nodePile;
+	*nodePile = list;
+#ifndef OPENMP
+   	setSingleNodeStatus(node, true);
+#endif
 }
 
-static void unlockMemorizedNodes()
+static inline boolean isNodeMemorized(Node * node, SmallNodeList * nodePile)
 {
-	SmallNodeList *list;
+#ifdef OPENMP
+	/* SF TODO There must be a faster way to do this: bit mask, hash table, tree, ... ? */
+	SmallNodeList * list;
 
-	while (nodePile) {
-		list = nodePile;
-		nodePile = list->next;
-		setSingleNodeStatus(list->node, false);
+	for (list = nodePile; list; list = list->next)
+		if (list->node == node)
+			return true;
+
+	return false;
+#else
+	return getNodeStatus(node);
+#endif
+}
+
+static void unMemorizeNodes(SmallNodeList ** nodePile)
+{
+	SmallNodeList * list;
+
+	while (*nodePile) {
+		list = *nodePile;
+		*nodePile = list->next;
+#ifndef OPENMP
+   		setSingleNodeStatus(list->node, false);
+#endif
 		deallocateSmallNodeList(list);
 	}
 }
@@ -326,7 +486,7 @@ static KmerOccurenceTable *referenceGraphKmers(char *preGraphFilename,
 
 	// Count kmers
 	velvetLog("Scanning pre-graph file %s for k-mers\n",
-	       preGraphFilename);
+		  preGraphFilename);
 
 	// First  line
 	if (!fgets(line, maxline, file))
@@ -476,7 +636,8 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 					    ReferenceMapping * referenceMappings,
 					    Coordinate referenceMappingCount,
 					    IDnum refCount,
-					    FILE ** rdmapFile,
+					    Annotation * annotations,
+					    IDnum annotationCount,
 					    boolean second_in_pair)
 {
 	Kmer word;
@@ -490,89 +651,35 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 	ReferenceMapping * refMap = NULL;
 	Coordinate uniqueIndex = 0;
 	Coordinate annotIndex = 0;
-	Coordinate annotStart = -1;
-	Coordinate annotPosition = -1;
-	Coordinate annotLength = -1;
-	IDnum annotSeqID = -1;
-	char line[MAXLINE];
-	long long_var;
-	long long longlong_var, longlong_var2, longlong_var3;
+	IDnum annotCount = 0;
 	boolean reversed;
+	SmallNodeList * nodePile = NULL;
+	Annotation * annotation = annotations;
 
 	Node *node;
 	Node *previousNode = NULL;
 
-	clearKmer(&word);
-	clearKmer(&antiWord);
-
 	// Neglect any read which will not be short paired
 	if ((!readTracking && category % 2 == 0)
-	    || category / 2 >= CATEGORIES) {
-		if (*rdmapFile) {
-			if (!fgets(line, MAXLINE, *rdmapFile)) {
-				fclose(*rdmapFile);
-				*rdmapFile = NULL;
-			}
-			while (line[0] != 'R') {
-				if (!fgets(line, MAXLINE, *rdmapFile)) {
-					fclose(*rdmapFile);
-					*rdmapFile = NULL;
-					break;
-				}
-			}
-		}
+	    || category / 2 >= CATEGORIES)
 		return;
-	}
 
-	// Get initial annotation
-	if (*rdmapFile) {
-		if (!fgets(line, MAXLINE, *rdmapFile)) {
-			fclose(*rdmapFile);
-			*rdmapFile = NULL;
-		}
-		else if (line[0] == 'R') 
-			;
-		else {
-			sscanf(line, "%ld\t%lld\t%lld\t%lld\n", &long_var,
-			       &longlong_var, &longlong_var2, &longlong_var3);
-			annotSeqID = (IDnum) long_var;
-
-			annotPosition = (Coordinate) longlong_var;
-			annotStart = (Coordinate) longlong_var2;
-			if (annotSeqID > 0)
-				annotLength = (Coordinate) longlong_var3 - annotStart;
-			else
-				annotLength = annotStart - (Coordinate) longlong_var3;
-			annotIndex = 0;
-		}
-	}
-	
 	// Neglect any string shorter than WORDLENGTH :
-	if (getLength(tString) < wordLength) {
-		if (*rdmapFile) {
-			while (line[0] != 'R') {
-				if (!fgets(line, MAXLINE, *rdmapFile)) {
-					fclose(*rdmapFile);
-					*rdmapFile = NULL;
-					break;
-				}
-			}
-		}
+	if (getLength(tString) < wordLength)
 		return;
-	}
 
 	// Verify that all short reads are reasonnably short
 	if (getLength(tString) > USHRT_MAX) {
 		velvetLog("Short read of length %lli, longer than limit %i\n",
-		       (long long) getLength(tString), SHRT_MAX);
+			  (long long) getLength(tString), SHRT_MAX);
 		velvetLog("You should better declare this sequence as long, because it genuinely is!\n");
 		exit(1);
 	}
-	// Allocate memory for the read pairs
-	if (!readStartsAreActivated(graph))
-		activateReadStarts(graph);
 
-	// Fill in the initial word : 
+	clearKmer(&word);
+	clearKmer(&antiWord);
+
+	// Fill in the initial word :
 	for (readNucleotideIndex = 0;
 	     readNucleotideIndex < wordLength - 1; readNucleotideIndex++) {
 		nucleotide = getNucleotide(readNucleotideIndex, tString);
@@ -600,35 +707,19 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 		}
 
 		// Update annotation if necessary
-		if (*rdmapFile && line[0] != 'R' && annotIndex == annotLength) {
-			if (!fgets(line, MAXLINE, *rdmapFile)) {
-				fclose(*rdmapFile);
-				*rdmapFile = NULL;
-			}
-			else if (line[0] == 'R') 
-				;
-			else {
-				sscanf(line, "%ld\t%lld\t%lld\t%lld\n", &long_var,
-				       &longlong_var, &longlong_var2, &longlong_var3);
-				annotSeqID = (IDnum) long_var;
-
-				annotPosition = (Coordinate) longlong_var;
-				annotStart = (Coordinate) longlong_var2;
-				if (annotSeqID > 0)
-					annotLength = (Coordinate) longlong_var3 - annotStart;
-				else
-					annotLength = annotStart - (Coordinate) longlong_var3;
-				annotIndex = 0;
-			}
+		if (annotCount < annotationCount && annotIndex == getAnnotationLength(annotation)) {
+			annotation = getNextAnnotation(annotation);
+			annotCount++;
 		}
 
 		// Search for reference mapping
-		if (*rdmapFile && line[0] != 'R' && uniqueIndex >= annotPosition && annotSeqID <= refCount && annotSeqID >= -refCount) {
-			refID = annotSeqID;
+		if (annotCount < annotationCount && uniqueIndex >= getPosition(annotation))
+		{
+			refID = getAnnotSequenceID(annotation);
 			if (refID > 0)
-				refCoord = annotStart + annotIndex;
+				refCoord = getStart(annotation) + annotIndex;
 			else
-				refCoord = annotStart - annotIndex;
+				refCoord = getStart(annotation) - annotIndex;
 			
 			refMap = findReferenceMapping(refID, refCoord, referenceMappings, referenceMappingCount);
 			// If success
@@ -642,7 +733,7 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 				if (previousNode)
 					break;
 			}
-		}		
+		}
 		// if not.. look in table
 		else {
 			reversed = false;
@@ -683,7 +774,7 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 
 		}
 
-		if (*rdmapFile && line[0] != 'R' && uniqueIndex >= annotPosition)
+		if (annotCount < annotationCount && uniqueIndex >= getPosition(annotation))
 			annotIndex++;
 		else
 			uniqueIndex++;
@@ -691,23 +782,20 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 		previousNode = node;
 
 		// Fill in graph
-		if (node && !getNodeStatus(node)) {
+		if (node && !isNodeMemorized(node, nodePile))
+		{
+#ifdef OPENMP
+			lockNode(node);
+#endif
 			incrementReadStartCount(node, graph);
-			setSingleNodeStatus(node, true);
-			memorizeNode(node);
+#ifdef OPENMP
+			unLockNode(node);
+#endif
+			memorizeNode(node, &nodePile);
 		}
 	}
 
-	unlockMemorizedNodes();
-	if (*rdmapFile) {
-		while (line[0] != 'R') {
-			if (!fgets(line, MAXLINE, *rdmapFile)) {
-				fclose(*rdmapFile);
-				*rdmapFile = NULL;
-				break;
-			}
-		}
-	}
+	unMemorizeNodes(&nodePile);
 }
 
 static void threadSequenceThroughGraph(TightString * tString,
@@ -719,7 +807,8 @@ static void threadSequenceThroughGraph(TightString * tString,
 				       ReferenceMapping * referenceMappings,
 				       Coordinate referenceMappingCount,
 				       IDnum refCount,
-				       FILE ** rdmapFile,
+				       Annotation * annotations,
+				       IDnum annotationCount,
 				       boolean second_in_pair)
 {
 	Kmer word;
@@ -731,7 +820,7 @@ static void threadSequenceThroughGraph(TightString * tString,
 
 	PassageMarkerI marker = NULL_IDX;
 	PassageMarkerI previousMarker = NULL_IDX;
-	Node *node;
+	Node *node = NULL;
 	Node *previousNode = NULL;
 	Coordinate coord = 0;
 	Coordinate previousCoord = 0;
@@ -741,57 +830,20 @@ static void threadSequenceThroughGraph(TightString * tString,
 	IDnum refID;
 	Coordinate refCoord = 0;
 	ReferenceMapping * refMap;
+	Annotation * annotation = annotations;
 	Coordinate index = 0;
 	Coordinate uniqueIndex = 0;
 	Coordinate annotIndex = 0;
-	Coordinate annotStart = -1;
-	Coordinate annotPosition = -1;
-	Coordinate annotLength = -1;
-	IDnum annotSeqID = 0;
-	char line[MAXLINE];
-	long long_var;
-	long long longlong_var, longlong_var2, longlong_var3;
+	IDnum annotCount = 0;
+	SmallNodeList * nodePile = NULL;
+
+	// Neglect any string shorter than WORDLENGTH :
+	if (getLength(tString) < wordLength)
+		return;
 
 	clearKmer(&word);
 	clearKmer(&antiWord);
 
-	// Get initial annotation
-	if (*rdmapFile) {
-		if (!fgets(line, MAXLINE, *rdmapFile)) {
-			fclose(*rdmapFile);
-			*rdmapFile = NULL;
-		}
-		else if (line[0] == 'R') 
-			;
-		else {
-			sscanf(line, "%ld\t%lld\t%lld\t%lld\n", &long_var,
-			       &longlong_var, &longlong_var2, &longlong_var3);
-			annotSeqID = (IDnum) long_var;
-
-			annotPosition = (Coordinate) longlong_var;
-			annotStart = (Coordinate) longlong_var2;
-			if (annotSeqID > 0)
-				annotLength = (Coordinate) longlong_var3 - annotStart;
-			else
-				annotLength = annotStart - (Coordinate) longlong_var3;
-			annotIndex = 0;
-		}
-	}
-
-	// Neglect any string shorter than WORDLENGTH :
-	if (getLength(tString) < wordLength) {
-		if (*rdmapFile) {
-			while (line[0] != 'R') {
-				if (!fgets(line, MAXLINE, *rdmapFile)) {
-					fclose(*rdmapFile);
-					*rdmapFile = NULL;
-					break;
-				}
-			}
-		}
-		return;
-	}
-
 	// Fill in the initial word : 
 	for (readNucleotideIndex = 0;
 	     readNucleotideIndex < wordLength - 1; readNucleotideIndex++) {
@@ -819,26 +871,9 @@ static void threadSequenceThroughGraph(TightString * tString,
 		}
 
 		// Update annotation if necessary
-		if (*rdmapFile && line[0] != 'R' && annotIndex == annotLength) {
-			if (!fgets(line, MAXLINE, *rdmapFile)) {
-				fclose(*rdmapFile);
-				*rdmapFile = NULL;
-			}
-			else if (line[0] == 'R') 
-				;
-			else {
-				sscanf(line, "%ld\t%lld\t%lld\t%lld\n", &long_var,
-				       &longlong_var, &longlong_var2, &longlong_var3);
-				annotSeqID = (IDnum) long_var;
-
-				annotPosition = (Coordinate) longlong_var;
-				annotStart = (Coordinate) longlong_var2;
-				if (annotSeqID > 0)
-					annotLength = (Coordinate) longlong_var3 - annotStart;
-				else
-					annotLength = annotStart - (Coordinate) longlong_var3;
-				annotIndex = 0;
-			}
+		if (annotCount < annotationCount && annotIndex == getAnnotationLength(annotation)) {
+			annotation = getNextAnnotation(annotation);
+			annotCount++;
 		}
 
 		// Search for reference mapping
@@ -862,12 +897,12 @@ static void threadSequenceThroughGraph(TightString * tString,
 			}
 		}
 		// Search for reference-based mapping
-		else if (*rdmapFile && line[0] != 'R' && uniqueIndex >= annotPosition && annotSeqID <= refCount && annotSeqID >= -refCount) {
-			refID = annotSeqID;
+		else if (annotCount < annotationCount && uniqueIndex >= getPosition(annotation) && getAnnotSequenceID(annotation) <= refCount && getAnnotSequenceID(annotation) >= -refCount) {
+			refID = getAnnotSequenceID(annotation);
 			if (refID > 0)
-				refCoord = annotStart + annotIndex; 
+				refCoord = getStart(annotation) + annotIndex; 
 			else
-				refCoord = annotStart - annotIndex; 
+				refCoord = getStart(annotation) - annotIndex; 
 			
 			refMap = findReferenceMapping(refID, refCoord, referenceMappings, referenceMappingCount);
 			// If success
@@ -936,13 +971,17 @@ static void threadSequenceThroughGraph(TightString * tString,
 		}
 
 		// Increment positions
-		if (*rdmapFile && line[0] != 'R' && uniqueIndex >= annotPosition) 
+		if (annotCount < annotationCount && uniqueIndex >= getPosition(annotation)) 
 			annotIndex++;
 		else
 			uniqueIndex++;
 
 		// Fill in graph
-		if (node) {
+		if (node)
+		{
+#ifdef OPENMP
+			lockNode(node);
+#endif
 			kmerIndex = readNucleotideIndex - wordLength;
 
 			if (previousNode == node
@@ -961,7 +1000,9 @@ static void threadSequenceThroughGraph(TightString * tString,
 					incrementOriginalVirtualCoverage
 					    (node, category / 2, 1);
 				}
-
+#ifdef OPENMP
+				unLockNode(node);
+#endif
 			} else {
 				if (category / 2 >= CATEGORIES) {
 					marker =
@@ -980,15 +1021,13 @@ static void threadSequenceThroughGraph(TightString * tString,
 					previousMarker = marker;
 				} else {
 					if (readTracking) {
-						if (!getNodeStatus(node)) {
+						if (!isNodeMemorized(node, nodePile)) {
 							addReadStart(node,
 								     seqID,
 								     coord,
 								     graph,
 								     kmerIndex);
-							setSingleNodeStatus
-							    (node, true);
-							memorizeNode(node);
+							memorizeNode(node, &nodePile);
 						} else {
 							blurLastShortReadMarker
 							    (node, graph);
@@ -1001,106 +1040,148 @@ static void threadSequenceThroughGraph(TightString * tString,
 					incrementOriginalVirtualCoverage
 					    (node, category / 2, 1);
 				}
-
+#ifdef OPENMP
+				lockTwoNodes(node, previousNode);
+#endif
 				createArc(previousNode, node, graph);
+#ifdef OPENMP
+				unLockTwoNodes(node, previousNode);
+#endif
 			}
 
 			previousNode = node;
 			previousCoord = coord;
 		}
-
 		index++;
 	}
 
-	unlockMemorizedNodes();
-	if (*rdmapFile) {
-		while (line[0] != 'R') {
-			if (!fgets(line, MAXLINE, *rdmapFile)) {
-				fclose(*rdmapFile);
-				*rdmapFile = NULL;
-				break;
-			}
-		}
-	}
+	if (readTracking && category / 2 < CATEGORIES)
+		unMemorizeNodes(&nodePile);
 }
 
 static void fillUpGraph(ReadSet * reads,
-			KmerOccurenceTable * kmerTable, Graph * graph,
-			boolean readTracking, boolean double_strand,
-			ReferenceMapping * referenceMappings, Coordinate referenceMappingCount,
+			KmerOccurenceTable * kmerTable,
+			Graph * graph,
+			boolean readTracking,
+			boolean double_strand,
+			ReferenceMapping * referenceMappings,
+			Coordinate referenceMappingCount,
 			IDnum refCount,
 			char * roadmapFilename)
 {
 	IDnum readIndex;
-	Category category;
-	FILE * file = NULL;
-	char line[MAXLINE];
-	boolean second_in_pair = false;
+	RoadMapArray *roadmap = NULL;
+	Coordinate *annotationOffset = NULL;
+	struct timeval start, end, diff;
 	
-	if (referenceMappings) {
-	    file = fopen(roadmapFilename, "r");
-	    while(fgets(line, MAXLINE, file))
-		    if (line[0] == 'R')
-			    break;
+	if (referenceMappings)
+	{
+		roadmap = importReferenceRoadMapArray(roadmapFilename);
+		annotationOffset = callocOrExit(reads->readCount, Coordinate);
+		for (readIndex = 1; readIndex < reads->readCount; readIndex++)
+			annotationOffset[readIndex] = annotationOffset[readIndex - 1]
+						      + getAnnotationCount(getRoadMapInArray(roadmap, readIndex - 1));
 	}
 
 	resetNodeStatus(graph);
+	// Allocate memory for the read pairs
+	if (!readStartsAreActivated(graph))
+		activateReadStarts(graph);
+
+	gettimeofday(&start, NULL);
+#ifdef OPENMP
+	initSmallNodeListMemory();
+	createNodeLocks(graph);
+	#pragma omp parallel for
+#endif
+	for (readIndex = refCount; readIndex < reads->readCount; readIndex++)
+	{
+		Annotation * annotations = NULL;
+		IDnum annotationCount = 0;
+		Category category;
+		boolean second_in_pair;
+
+		if (readIndex % 1000000 == 0)
+			velvetLog("Ghost Threading through reads %ld / %ld\n",
+				  (long) readIndex, (long) reads->readCount);
 
-	for (readIndex = 0; readIndex < reads->readCount; readIndex++) {
 		category = reads->categories[readIndex];
+		second_in_pair = reads->categories[readIndex] & 1 && isSecondInPair(reads, readIndex);
+
+		if (referenceMappings)
+		{
+			annotationCount = getAnnotationCount(getRoadMapInArray(roadmap, readIndex));
+			annotations = getAnnotationInArray(roadmap->annotations, annotationOffset[readIndex]);
+		}
 	
 		ghostThreadSequenceThroughGraph(getTightStringInArray(reads->tSequences, readIndex),
 						kmerTable,
 						graph, readIndex + 1,
-						category, 
+						category,
 						readTracking, double_strand,
 						referenceMappings, referenceMappingCount,
-					  	refCount, &file,
+					  	refCount, annotations, annotationCount,
 						second_in_pair);
-
-		if (category % 2) 
-			second_in_pair = (second_in_pair? false : true);
-		else 
-			second_in_pair = false;
 	}
-
 	createNodeReadStartArrays(graph);
+	gettimeofday(&end, NULL);
+	timersub(&end, &start, &diff);
+	velvetLog(" === Ghost-Threaded in %ld.%06ld s\n", diff.tv_sec, diff.tv_usec);
 
-	if (referenceMappings) {
-	    file = fopen(roadmapFilename, "r");
-	    while(fgets(line, MAXLINE, file))
-		    if (line[0] == 'R')
-			    break;
-	}
+	gettimeofday(&start, NULL);
+#ifdef OPENMP
+	int threads = omp_get_max_threads();
+	if (threads > 32)
+		threads = 32;
 
-	second_in_pair = false;
-	for (readIndex = 0; readIndex < reads->readCount; readIndex++) {
-		category = reads->categories[readIndex];
-	
-		if (readIndex % 100000 == 0)
+	#pragma omp parallel for num_threads(threads)
+#endif
+	for (readIndex = 0; readIndex < reads->readCount; readIndex++)
+	{
+		Annotation * annotations = NULL;
+		IDnum annotationCount = 0;
+		Category category;
+		boolean second_in_pair;
+
+		if (readIndex % 1000000 == 0)
 			velvetLog("Threading through reads %li / %li\n",
-			       (long) readIndex, (long) reads->readCount);
+				  (long) readIndex, (long) reads->readCount);
+
+		category = reads->categories[readIndex];
+		second_in_pair = reads->categories[readIndex] % 2 && isSecondInPair(reads, readIndex);
+
+		if (referenceMappings)
+		{
+			annotationCount = getAnnotationCount(getRoadMapInArray(roadmap, readIndex));
+			annotations = getAnnotationInArray(roadmap->annotations, annotationOffset[readIndex]);
+		}
 
 		threadSequenceThroughGraph(getTightStringInArray(reads->tSequences, readIndex),
 					   kmerTable,
 					   graph, readIndex + 1, category,
 					   readTracking, double_strand,
 					   referenceMappings, referenceMappingCount,
-					   refCount, &file, second_in_pair);
+					   refCount, annotations, annotationCount, second_in_pair);
+	}
+	gettimeofday(&end, NULL);
+	timersub(&end, &start, &diff);
+	velvetLog(" === Threaded in %ld.%06ld s\n", diff.tv_sec, diff.tv_usec);
 
-		if (category % 2) 
-			second_in_pair = (second_in_pair? false : true);
-		else 
-			second_in_pair = false;
+#ifdef OPENMP
+	free(nodeLocks);
+	nodeLocks = NULL;
+#endif
+
+	if (referenceMappings)
+	{
+		destroyRoadMapArray(roadmap);
+		free (annotationOffset);
 	}
 
 	orderNodeReadStartArrays(graph);
 
-	if (smallNodeListMemory != NULL)
-		destroyRecycleBin(smallNodeListMemory);
+	destroySmallNodeListMemmory();
 
-	if (file) 
-		fclose(file);
 	destroyKmerOccurenceTable(kmerTable);
 }
 
