@@ -33,41 +33,6 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 #include "fibHeap.h"
 #include "utility.h"
 
-static void surveyPath(PassageMarkerI marker)
-{
-	Coordinate length = 0;
-	Coordinate realLength = 0;
-	PassageMarkerI current = marker;
-
-	if (passageMarkerDirection(current) < 0)
-		current = getTwinMarker(current);
-
-	for (; current != NULL_IDX; current = getNextInSequence(current)) {
-		length +=
-		    getNodeLength(getNode(current)) -
-		    getStartOffset(current) - getFinishOffset(current);
-		if (getPassageMarkerFinish(current) > 0)
-			realLength = getPassageMarkerFinish(current);
-	}
-
-	velvetLog("SURVEY %ld %lld %lld\n", (long) getAbsolutePassMarkerSeqID(marker),
-	       (long long) realLength, (long long) length);
-}
-
-void surveyPaths(Graph * graph)
-{
-	IDnum ID;
-	PassageMarkerI marker;
-
-	for (ID = 1; ID <= nodeCount(graph); ID++)
-		for (marker = getMarker(getNodeInGraph(graph, ID));
-		     marker != NULL_IDX; marker = getNextInNode(marker))
-			if ((passageMarkerDirection(marker) > 0
-			     && isInitial(marker))
-			    || (passageMarkerDirection(marker) < 0
-				&& isTerminal(marker)))
-				surveyPath(marker);
-}
 
 static PassageMarkerList *copyMarkers(Node * node)
 {
@@ -201,82 +166,6 @@ Coordinate computeVirtualNodeLength(Node * node)
 	return virtualLength;
 }
 
-void testForBizarreMarkers(Graph * graph)
-{
-	IDnum index;
-	Node *node;
-	PassageMarkerI marker;
-
-	for (index = 1; index < nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-		for (marker = getMarker(node); marker != NULL_IDX;
-		     marker = getNextInNode(marker)) {
-			if (getTwinMarker(marker) == NULL_IDX) 
-				exitErrorf(EXIT_FAILURE, false, "Bizarre marker %s",
-				       readPassageMarker(marker));
-		}
-	}
-}
-
-// COunts how many nodes are dead-ends
-IDnum countSinksAndSources(Graph * graph)
-{
-	IDnum nodeIndex;
-	IDnum result = 0;
-	Node *node;
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		node = getNodeInGraph(graph, nodeIndex);
-		if (arcCount(node) == 0
-		    || arcCount(getTwinNode(node)) == 0)
-			result++;
-	}
-
-	return result;
-}
-
-// Counts how many nodes have several arcs either going in or coming out
-IDnum countTangles(Graph * graph)
-{
-	IDnum nodeIndex;
-	IDnum result = 0;
-	Node *node;
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		node = getNodeInGraph(graph, nodeIndex);
-		if (arcCount(node) > 1 || arcCount(getTwinNode(node)) > 1)
-			result++;
-	}
-	return result;
-}
-
-// Counts nodes with exactly one incoming and one outgoing arc
-IDnum countRepeats(Graph * graph)
-{
-	IDnum nodeIndex;
-	IDnum result = 0;
-	Node *node;
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		node = getNodeInGraph(graph, nodeIndex);
-		if (arcCount(node) == 1
-		    && arcCount(getTwinNode(getDestination(getArc(node))))
-		    == 1)
-			if (getNodeID
-			    (getTwinNode(getDestination(getArc(node)))) < 0
-			    ||
-			    getNodeID(getTwinNode
-				      (getDestination(getArc(node)))) >
-			    getNodeID(node))
-				result++;
-	}
-	return result;
-
-}
-
 // Counts the number of markers for one node
 int nodeGenomicMultiplicity(Node * node, IDnum firstStrain)
 {
@@ -294,95 +183,6 @@ int nodeGenomicMultiplicity(Node * node, IDnum firstStrain)
 	return counter;
 }
 
-// Counts the number of markers for one node
-IDnum nodeMultiplicity(Node * node)
-{
-	int counter = 0;
-	PassageMarkerI marker;
-
-	if (node == NULL)
-		return 0;
-
-	marker = getMarker(node);
-	while (marker != NULL_IDX) {
-		counter++;
-		marker = getNextInNode(marker);
-	}
-	return counter;
-}
-
-// Prints out a set of predefined statistics for one node
-char *nodeStatistics(Node * node)
-{
-	char *s = mallocOrExit(100, char);
-	sprintf(s, "NODE %ld\t%lld\t%i\t%i\t%ld", (long) getNodeID(node),
-		(long long) getNodeLength(node), simpleArcCount(node),
-		simpleArcCount(getTwinNode(node)), (long) nodeMultiplicity(node));
-	return s;
-}
-
-// Prints out a table of statistics for all the nodes of the graph
-void displayGraphStatistics(Graph * graph)
-{
-	IDnum nodeIndex;
-	velvetLog("NODE ID\tlgth\tFwd\tBck\tMult\n");
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++)
-		velvetLog("%s\n",
-		       nodeStatistics(getNodeInGraph(graph, nodeIndex)));
-}
-
-void displayNodeStatisticsSelective(Node * node, IDnum first)
-{
-	PassageMarkerI marker;
-	boolean originalGenome;
-	boolean strain;
-
-	if (node == NULL)
-		return;
-
-	marker = getMarker(node);
-
-	originalGenome = false;
-	strain = false;
-	while (marker != NULL_IDX) {
-		if (getAbsolutePassMarkerSeqID(marker) < first)
-			originalGenome = true;
-
-		if (getAbsolutePassMarkerSeqID(marker) >= first)
-			strain = true;
-
-		marker = getNextInNode(marker);
-	}
-
-	velvetLog("%s", nodeStatistics(node));
-	if (originalGenome && !strain)
-		velvetLog("\tTRUE");
-	else
-		velvetLog("\tFALSE");
-
-	if (originalGenome && strain)
-		velvetLog("\tTRUE");
-	else
-		velvetLog("\tFALSE");
-
-
-	if (strain && !originalGenome)
-		velvetLog("\tTRUE\n");
-	else
-		velvetLog("\tFALSE\n");
-
-}
-
-void displayGraphStatisticsSelective(Graph * graph, IDnum first)
-{
-	IDnum index;
-
-	for (index = 1; index <= nodeCount(graph); index++)
-		displayNodeStatisticsSelective(getNodeInGraph
-					       (graph, index), first);
-
-}
-
 boolean isOnlyGenome(Node * node, IDnum firstStrain)
 {
 	PassageMarkerI marker;
@@ -451,24 +251,6 @@ boolean isSNP(Node * node, IDnum firstStrain, int WORDLENGTH)
 	return true;
 }
 
-IDnum strainMarkerCount(Node * node, IDnum firstStrain)
-{
-	PassageMarkerI marker;
-	IDnum counter = 0;
-
-	for (marker = getMarker(node); marker != NULL_IDX;
-	     marker = getNextInNode(marker))
-		if (getAbsolutePassMarkerSeqID(marker) >= firstStrain)
-			counter++;
-
-	return counter;
-}
-
-boolean isError(Node * node, IDnum firstStrain)
-{
-	return (strainMarkerCount(node, firstStrain) < 5);
-}
-
 void removeStrainMarkers(Node * node, IDnum firstStrain)
 {
 	PassageMarkerI marker;
@@ -485,64 +267,6 @@ void removeStrainMarkers(Node * node, IDnum firstStrain)
 
 }
 
-void chainSawCorrection(Graph * graph, int minMult)
-{
-	IDnum nodeIndex;
-	IDnum removed = 0;
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		if (markerCount(getNodeInGraph(graph, nodeIndex)) <
-		    minMult) {
-			destroyNode(getNodeInGraph(graph, nodeIndex),
-				    graph);
-			removed++;
-		}
-	}
-
-	velvetLog("%li dubious nodes removed\n", (long) removed);
-	concatenateGraph(graph);
-	velvetLog("%li node in the end\n", (long) nodeCount(graph));
-}
-
-void grossErrorRemoval(Graph * graph, IDnum firstStrain)
-{
-	IDnum nodeIndex;
-	IDnum removed = 0;
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		if (isError(getNodeInGraph(graph, nodeIndex), firstStrain)) {
-			if (isOnlyStrain
-			    (getNodeInGraph(graph, nodeIndex),
-			     firstStrain)) {
-				destroyNode(getNodeInGraph
-					    (graph, nodeIndex), graph);
-				removed++;
-			} else
-				removeStrainMarkers(getNodeInGraph
-						    (graph, nodeIndex),
-						    firstStrain);
-		}
-	}
-
-	velvetLog("%li dubious nodes removed\n", (long) removed);
-	concatenateGraph(graph);
-	velvetLog("%li node in the end\n", (long) nodeCount(graph));
-}
-
-IDnum countSNPs(Graph * graph, IDnum firstStrain, int WORDLENGTH)
-{
-	IDnum index;
-	IDnum counter = 0;
-
-	for (index = 1; index < nodeCount(graph); index++)
-		if (isSNP
-		    (getNodeInGraph(graph, index), firstStrain,
-		     WORDLENGTH))
-			counter++;
-
-	return counter;
-}
-
 Coordinate commonLength(Node * node, IDnum firstStrain)
 {
 	PassageMarkerI marker = getMarker(node);
@@ -563,19 +287,6 @@ Coordinate commonLength(Node * node, IDnum firstStrain)
 	return (Coordinate) orig *getNodeLength(node);
 }
 
-Coordinate countCommonLength(Graph * graph, IDnum firstStrain)
-{
-	IDnum index;
-	Coordinate res = 0;
-
-	for (index = 1; index <= nodeCount(graph); index++)
-		res +=
-		    commonLength(getNodeInGraph(graph, index),
-				 firstStrain);
-
-	return res;
-}
-
 boolean isMixed(Node * node, IDnum firstStrain)
 {
 	return !isOnlyStrain(node, firstStrain)
@@ -641,79 +352,6 @@ int countLocalBreakpoints(PassageMarkerI marker, IDnum firstStrain)
 	return counter;
 }
 
-IDnum countBreakpoints(Graph * graph, IDnum firstStrain)
-{
-	PassageMarkerI marker;
-	IDnum seqIndex;
-	IDnum total = 0;
-
-	for (seqIndex = 1; seqIndex < firstStrain; seqIndex++) {
-		marker = getMarker(getNodeInGraph(graph, seqIndex));
-		while (marker != NULL_IDX) {
-			total +=
-			    countLocalBreakpoints(marker, firstStrain);
-			marker = getNextInSequence(marker);
-		}
-	}
-
-	return total;
-}
-
-IDnum countStrainOnlyNodes(Graph * graph, IDnum firstStrain)
-{
-	IDnum index;
-	IDnum total = 0;
-
-	for (index = 1; index <= nodeCount(graph); index++)
-		if (isOnlyStrain
-		    (getNodeInGraph(graph, index), firstStrain))
-			total++;
-
-	return total;
-}
-
-Coordinate countStrainOnlyBp(Graph * graph, IDnum firstStrain)
-{
-	IDnum index;
-	Coordinate total = 0;
-	Node *node;
-	Arc *arc;
-	Coordinate local;
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		if (isOnlyStrain
-		    (getNodeInGraph(graph, index), firstStrain)) {
-			node = getNodeInGraph(graph, index);
-			local = getNodeLength(node);
-
-			for (arc = getArc(node); arc != NULL;
-			     arc = getNextArc(arc)) {
-				if (!isOnlyStrain
-				    (getDestination(arc), firstStrain)) {
-					local -= 24;
-					break;
-				}
-			}
-
-			for (arc = getArc(getTwinNode(node)); arc != NULL;
-			     arc = getNextArc(arc)) {
-				if (!isOnlyStrain
-				    (getDestination(arc), firstStrain)) {
-					local -= 24;
-					break;
-				}
-			}
-
-			if (local < 0)
-				local = 1;
-
-			total += local;
-		}
-	}
-
-	return total;
-}
-
 IDnum genomeMarkerCount(Node * node, IDnum firstStrain)
 {
 	PassageMarkerI marker;
@@ -777,28 +415,8 @@ Coordinate newReadCoverage(Node * node, IDnum firstStrain)
 	return sum;
 }
 
-IDnum readStarts(Node * node)
+static void printShortCounts(FILE * outfile, Node * node, Graph * graph, ReadSet * reads)
 {
-	PassageMarkerI marker;
-	IDnum sum = 0;
-
-	if (node == NULL)
-		return 0;
-
-	for (marker = getMarker(node); marker != NULL_IDX;
-	     marker = getNextInNode(marker)) {
-		if (getPassageMarkerSequenceID(marker) > 0
-		    && isInitial(marker))
-			sum++;
-		else if (getPassageMarkerSequenceID(marker) < 0
-			 && isTerminal(marker))
-			sum++;
-	}
-
-	return sum;
-}
-
-static void printShortCounts(FILE * outfile, Node * node, Graph * graph, ReadSet * reads) {
 	IDnum counts[CATEGORIES];
 	Category cat;
 	IDnum shortReadIndex;
@@ -896,132 +514,6 @@ void displayGeneralStatistics(Graph * graph, char *filename, ReadSet * reads)
 	fclose(outfile);
 }
 
-void destroyStrainSpecificIslands(Graph * graph, IDnum firstStrain)
-{
-	IDnum index;
-	Arc *arc;
-	boolean isModified = true;
-	Node *node;
-	IDnum counter = 0;
-
-	resetNodeStatus(graph);
-
-	velvetLog("Destroying disconnected strain specific sub-graphs\n");
-
-	// Mark all genomic nodes 
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (!isOnlyStrain(node, firstStrain))
-			setNodeStatus(node, true);
-	}
-
-	// Mark nodes connected to genomic nodes
-	while (isModified) {
-		isModified = false;
-
-		for (index = 1; index <= nodeCount(graph); index++) {
-			node = getNodeInGraph(graph, index);
-
-			if (getNodeStatus(node))
-				continue;
-
-			for (arc = getArc(node); arc != NULL;
-			     arc = getNextArc(arc)) {
-				if (getNodeStatus(getDestination(arc))) {
-					isModified = true;
-					setNodeStatus(node, true);
-				}
-			}
-
-			for (arc = getArc(getTwinNode(node)); arc != NULL;
-			     arc = getNextArc(arc)) {
-				if (getNodeStatus(getDestination(arc))) {
-					isModified = true;
-					setNodeStatus(node, true);
-				}
-			}
-		}
-	}
-
-	// Remove all unmarked nodes
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (!getNodeStatus(node)) {
-			destroyNode(node, graph);
-			counter++;
-		}
-	}
-
-	// Renumber graph nodes
-	velvetLog("Removed %li nodes \n", (long) counter);
-	renumberNodes(graph);
-}
-
-void displayStrainOnlySequences(Graph * graph, IDnum firstStrain,
-				char *inputFilename, char *filename,
-				int WORDLENGTH)
-{
-	IDnum nodeIndex;
-	Node *node;
-	FILE *outfile = fopen(filename, "w");
-	Coordinate start, finish;
-	char str[100];
-	TightString *tString;
-	IDnum readID;
-	Coordinate readCoord;
-
-	if (outfile == NULL) {
-		velvetLog("Could not write into %s, sorry\n", filename);
-		return;
-	}
-
-	destroyStrainSpecificIslands(graph, firstStrain);
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		node = getNodeInGraph(graph, nodeIndex);
-		if (isOnlyStrain(node, firstStrain)
-		    && getNodeLength(node) > 500) {
-			tString = expandNode(node, WORDLENGTH);
-			readID =
-			    getPassageMarkerSequenceID(getMarker(node));
-			readCoord = getPassageMarkerStart(getMarker(node));
-			velvetFprintf(outfile, "> UNIQUE SEQUENCE %ld; %lld\n",
-				(long) readID, (long long) readCoord);
-
-			start = 0;
-			while (start <= getLength(tString)) {
-				finish = start + 60;
-				readTightStringFragment(tString, start,
-							finish, str);
-				velvetFprintf(outfile, "%s\n", str);
-				start = finish;
-			}
-		}
-	}
-
-	fclose(outfile);
-}
-
-void displayStrainOnlyDescriptors(Graph * graph, IDnum firstStrain)
-{
-	IDnum nodeIndex;
-	Node *node;
-	char *str;
-
-	destroyStrainSpecificIslands(graph, firstStrain);
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		velvetLog("node %li from %li\n", (long) nodeIndex, (long) nodeCount(graph));
-		node = getNodeInGraph(graph, nodeIndex);
-
-		if (isOnlyStrain(node, firstStrain)) {
-			str = readNode(node);
-			velvetLog("> UNIQUE SEQUENCE %s\n", str);
-			free(str);
-		}
-	}
-}
-
 void displayLocalBreakpoint(PassageMarkerI strainMarker,
 			    IDnum firstStrain,
 			    PassageMarkerI genomeMarker,
@@ -1157,68 +649,6 @@ void displayLocalBreakpoint(PassageMarkerI strainMarker,
 //              velvetLog("stopped 1\n");
 }
 
-void displayBreakpoints(Graph * graph, IDnum firstStrain)
-{
-	IDnum nodeIndex;
-	Node *node;
-	PassageMarkerI strainMarker, genomeMarker;
-	Node **genomeDestination =
-	    callocOrExit(2 * nodeCount(graph) + 1, Node *);
-	Node **strainDestination =
-	    callocOrExit(2 * nodeCount(graph) + 1, Node *);
-	IDnum counters[3];
-
-	counters[0] = 0;
-	counters[1] = 0;
-	counters[2] = 0;
-
-	resetNodeStatus(graph);
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		node = getNodeInGraph(graph, nodeIndex);
-
-		if (arcCount(node) <= 1
-		    && arcCount(getTwinNode(node)) <= 1) {
-			continue;
-		}
-
-		if (nodeGenomicMultiplicity(node, firstStrain) != 1) {
-			continue;
-		}
-
-		if (isOnlyGenome(node, firstStrain)) {
-			continue;
-		}
-
-		for (genomeMarker = getMarker(node); genomeMarker != NULL_IDX;
-		     genomeMarker = getNextInNode(genomeMarker))
-			if (getAbsolutePassMarkerSeqID(genomeMarker) <
-			    firstStrain)
-				break;
-
-		// Go through all strain passage marker
-		for (strainMarker = getMarker(node); strainMarker != NULL_IDX;
-		     strainMarker = getNextInNode(strainMarker)) {
-			displayLocalBreakpoint(strainMarker, firstStrain,
-					       genomeMarker,
-					       genomeDestination,
-					       strainDestination, counters,
-					       nodeCount(graph));
-			displayLocalBreakpoint(getTwinMarker(strainMarker),
-					       firstStrain,
-					       getTwinMarker(genomeMarker),
-					       genomeDestination,
-					       strainDestination, counters,
-					       nodeCount(graph));
-		}
-	}
-
-
-	velvetLog("%li\t%li\t%li\n", (long) counters[0], (long) counters[1], (long) counters[2]);
-	free(strainDestination);
-	free(genomeDestination);
-}
-
 PassageMarkerI genomeMarker(Node * node, IDnum firstStrain)
 {
 	PassageMarkerI marker;
@@ -1281,46 +711,6 @@ void projectGraphToFile(Graph * graph, char *filename, int WORDLENGTH,
 	fclose(outfile);
 }
 
-void removeReferenceMarkers(Graph * graph, IDnum firstStrain)
-{
-	IDnum ID;
-	Node *node;
-	PassageMarkerI marker, oldMarker;
-
-	for (ID = 1; ID <= nodeCount(graph); ID++) {
-		node = getNodeInGraph(graph, ID);
-		marker = getMarker(node);
-		while (marker != NULL_IDX) {
-			if (getAbsolutePassMarkerSeqID(marker) <
-			    firstStrain) {
-				if (!isInitial(marker))
-					changeMultiplicity
-					    (getArcBetweenNodes
-					     (getNode
-					      (getPreviousInSequence
-					       (marker)), node, graph),
-					     -1);
-				if (!isTerminal(marker))
-					changeMultiplicity
-					    (getArcBetweenNodes
-					     (node,
-					      getNode(getNextInSequence
-						      (marker)), graph),
-					     -1);
-				oldMarker = marker;
-				marker = getNextInNode(marker);
-				destroyPassageMarker(oldMarker);
-			} else
-				marker = getNextInNode(marker);
-		}
-
-		if (getMarker(node) == NULL_IDX)
-			destroyNode(node, graph);
-	}
-
-	concatenateGraph(graph);
-}
-
 static void exportLongNodeSequence(FILE * outfile, Node * node, Graph * graph) {
 	TightString *tString;
 	Coordinate position;
@@ -1402,53 +792,6 @@ void exportLongNodeSequences(char *filename, Graph * graph,
 	fclose(outfile);
 }
 
-/*
-void exportMediumNodeSequences(char* filename, Graph * graph, Coordinate minLength)
-{
-	IDnum dummy;
-	ReadSet *readSet = readFastAFile(sequenceFile);
-	char **reads = readSet->sequences;
-	TightString **sequences =
-	    newTightStringArrayFromStringArray(reads, dummy);
-	FILE *outfile = fopen(filename, "w");
-	char str[100];
-	IDnum nodeIndex;
-	TightString *tString;
-	Coordinate start, finish;
-	Node *node;
-	double sensitivity, specificity;
-
-	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
-		node = getNodeInGraph(graph, nodeIndex);
-		if (getNodeLength(node) < minLength
-		    || getNodeLength(node) >= maxLength)
-			continue;
-
-		tString = expandNode(node);
-		compareSequences(tString, sequences[0], &sensitivity,
-				 &specificity, WORDLENGTH);
-
-		velvetFprintf(outfile,
-			"> MEDIUM NODE %d, Sensitivity = %f, Specificity = %f\n",
-			nodeIndex, sensitivity, specificity);
-		velvetLog
-		    ("> MEDIUM NODE %d, Sensitivity = %f, Specificity = %f\n",
-		     nodeIndex, sensitivity, specificity);
-
-		start = 0;
-		while (start <= getLength(tString)) {
-			finish = start + 60;
-			readTightStringFragment(tString, start,
-						finish, str);
-			velvetFprintf(outfile, "%s\n", str);
-			start = finish;
-		}
-
-		destroyTightString(tString);
-	}
-}
-*/
-
 Coordinate maxLength(Graph * graph)
 {
 	IDnum index;
@@ -1605,485 +948,6 @@ double estimated_cov(Graph * graph, char * directory)
 	return -1;
 }
 
-static void destroyMixedNode(Node * node)
-{
-	PassageMarkerI marker = getMarker(node);
-	PassageMarkerI current;
-
-	while (marker != NULL_IDX) {
-		while (!isInitial(marker))
-			marker = getPreviousInSequence(marker);
-
-		while (marker != NULL_IDX) {
-			current = marker;
-			marker = getNextInSequence(marker);
-			destroyPassageMarker(current);
-		}
-
-		marker = getMarker(node);
-	}
-}
-
-void destroyMixedReads(Graph * graph, IDnum minCoverage)
-{
-	IDnum index;
-	Node *node;
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-		if (markerCount(node) < minCoverage)
-			destroyMixedNode(node);
-	}
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-		if (getMarker(node) == NULL_IDX)
-			destroyNode(node, graph);
-	}
-
-	concatenateGraph(graph);
-}
-
-static boolean isConnectedRead(PassageMarkerI marker)
-{
-	PassageMarkerI current;
-
-	for (current = marker; getNodeStatus(getNode(current));
-	     current = getNextInSequence(current))
-		if (isTerminal(current))
-			return false;
-
-	for (current = getTwinMarker(marker);
-	     getNodeStatus(getNode(current));
-	     current = getNextInSequence(current))
-		if (isTerminal(current))
-			return false;
-
-	return true;
-}
-
-static void destroyWholeRead(PassageMarkerI marker)
-{
-	PassageMarkerI current = marker;
-	PassageMarkerI next;
-
-	while (!isInitial(current))
-		current = getPreviousInSequence(current);
-
-	for (; current != NULL_IDX; current = next) {
-		next = getNextInSequence(current);
-		destroyPassageMarker(current);
-	}
-}
-
-static void cleanUpNode(Node * node, Graph * graph)
-{
-	Category cat;
-	Node *twin = getTwinNode(node);
-	PassageMarkerI marker, twinMarker;
-
-	for (cat = 0; cat < CATEGORIES; cat++)
-		setVirtualCoverage(node, cat, 0);
-
-	while (getArc(node) != NULL)
-		destroyArc(getArc(node), graph);
-	while (getArc(twin) != NULL)
-		destroyArc(getArc(twin), graph);
-
-	for (marker = getMarker(node); marker != NULL_IDX;
-	     marker = getNextInNode(marker)) {
-		twinMarker = getTwinMarker(marker);
-
-		if (getNode(getNextInSequence(marker)) != twin
-		    || getPassageMarkerSequenceID(marker) > 0)
-			createArc(node,
-				  getNode(getNextInSequence(marker)),
-				  graph);
-		if (getNode(getNextInSequence(twinMarker)) != node
-		    || getPassageMarkerSequenceID(twinMarker) > 0)
-			createArc(twin,
-				  getNode(getNextInSequence(twinMarker)),
-				  graph);
-	}
-}
-
-void destroySinglePoolNodes(Graph * graph)
-{
-	IDnum index;
-	Node *node;
-	PassageMarkerI marker, next;
-
-	velvetLog("Destroying single pool nodes\n");
-	resetNodeStatus(graph);
-
-	// Remove empty, single pool nodes, mark other single pool nodes
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-		if (getVirtualCoverage(node, 0) != 0
-		    && getVirtualCoverage(node, 1) != 0)
-			continue;
-
-		if (getMarker(node) == NULL_IDX)
-			destroyNode(node, graph);
-		else
-			setNodeStatus(node, true);
-	}
-
-	// Remove disconnected reads
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL || !getNodeStatus(node))
-			continue;
-
-		for (marker = getMarker(node); marker != NULL_IDX;
-		     marker = next) {
-			if (isConnectedRead(marker))
-				next = getNextInNode(marker);
-			else {
-				destroyWholeRead(marker);
-				next = getMarker(node);
-			}
-		}
-	}
-
-	// Remove empty, single pool nodes, review coverage of the others
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL || !getNodeStatus(node))
-			continue;
-
-		if (getMarker(node) == NULL_IDX)
-			destroyNode(node, graph);
-		else
-			cleanUpNode(node, graph);
-	}
-
-	velvetLog("Done\n");
-
-	concatenateGraph(graph);
-}
-
-void destroyShortTips(Graph * graph)
-{
-	IDnum index;
-	Node *node;
-	boolean modified = true;
-
-	velvetLog("Removing short tips\n");
-
-	while (modified) {
-		modified = false;
-		for (index = 1; index <= nodeCount(graph); index++) {
-			node = getNodeInGraph(graph, index);
-			if (node == NULL)
-				continue;
-
-			if (getArc(node) == NULL
-			    || getArc(getTwinNode(node)) == NULL) {
-				if (getNodeLength(node) < 500) {
-					modified = true;
-					destroyNode(node, graph);
-				}
-			}
-		}
-	}
-
-	velvetLog("Done\n");
-
-	concatenateGraph(graph);
-}
-
-static Coordinate connectDomain(Node * node)
-{
-	Coordinate result = getNodeLength(node);
-	Arc *arc;
-
-	if (getNodeStatus(node))
-		return 0;
-	setNodeStatus(node, true);
-
-	for (arc = getArc(node); arc != NULL; arc = getNextArc(arc))
-		result += connectDomain(getDestination(arc));
-	for (arc = getArc(getTwinNode(node)); arc != NULL;
-	     arc = getNextArc(arc))
-		result += connectDomain(getDestination(arc));
-
-	return result;
-
-}
-
-static void destroyConnectedDomain(Node * node, Graph * graph)
-{
-	Arc *arc;
-
-	if (getNodeStatus(node))
-		return;
-	setNodeStatus(node, true);
-
-	for (arc = getArc(node); arc != NULL; arc = getNextArc(arc))
-		destroyConnectedDomain(getDestination(arc), graph);
-	for (arc = getArc(getTwinNode(node)); arc != NULL;
-	     arc = getNextArc(arc))
-		destroyConnectedDomain(getDestination(arc), graph);
-
-	destroyNode(node, graph);
-
-}
-
-void destroyDisconnectedElements(Graph * graph)
-{
-	Node *node;
-	IDnum index;
-	Coordinate domainSize;
-	FibHeap *heap = newFibHeap();
-	Coordinate *domainSizes =
-	    callocOrExit(1 + nodeCount(graph), Coordinate);
-
-	resetNodeStatus(graph);
-
-	velvetLog("Destroying disconnected domains\n");
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL || getNodeStatus(node))
-			continue;
-		domainSize = connectDomain(node);
-		velvetLog("CONNECT\t%lld\n", (long long) domainSize);
-		insertNodeIntoHeap(heap, domainSize, node);
-		domainSizes[index] = domainSize;
-	}
-
-	resetNodeStatus(graph);
-
-	while (true) {
-		node = removeNextNodeFromHeap(heap);
-		if (node == NULL || domainSizes[getNodeID(node)] > 1200)
-			break;
-
-		destroyConnectedDomain(node, graph);
-	}
-
-
-	destroyHeap(heap);
-	free(domainSizes);
-	velvetLog("Done\n");
-
-	concatenateGraph(graph);
-}
-
-static Coordinate connectDomainNodeCount(Node * node)
-{
-	Coordinate result = 1;
-	Arc *arc;
-
-	if (getNodeStatus(node))
-		return 0;
-	setNodeStatus(node, true);
-
-	for (arc = getArc(node); arc != NULL; arc = getNextArc(arc))
-		result += connectDomain(getDestination(arc));
-	for (arc = getArc(getTwinNode(node)); arc != NULL;
-	     arc = getNextArc(arc))
-		result += connectDomain(getDestination(arc));
-
-	return result;
-
-}
-
-void measureTangleSizes(Graph * graph, Coordinate maxLength)
-{
-	Node *node;
-	IDnum index;
-	Coordinate domainSize;
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-		if (getNodeLength(node) >= maxLength)
-			destroyNode(node, graph);
-	}
-
-	renumberNodes(graph);
-	resetNodeStatus(graph);
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL || getNodeStatus(node))
-			continue;
-		domainSize = connectDomainNodeCount(node);
-		velvetLog("CONNECT\t%lld\n", (long long) domainSize);
-	}
-
-	velvetLog("Done\n");
-}
-
-void destroyEmptyNodes(Graph * graph)
-{
-	IDnum index;
-	Node *node;
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-		if (getMarker(node) == NULL_IDX)
-			destroyNode(node, graph);
-	}
-
-	concatenateGraph(graph);
-}
-
-static Coordinate getReadLength(PassageMarkerI marker)
-{
-	PassageMarkerI current;
-	Coordinate totalLength = 0;
-
-	for (current = marker; current != NULL_IDX;
-	     current = getNextInSequence(current))
-		totalLength += getPassageMarkerLength(current);
-
-	return totalLength;
-}
-
-static void destroyRead(PassageMarkerI marker)
-{
-	PassageMarkerI current, next;
-
-	for (current = marker; current != NULL_IDX; current = next) {
-		next = getNextInSequence(current);
-		destroyPassageMarker(current);
-	}
-}
-
-void removeShortReads(Graph * graph)
-{
-	IDnum index;
-	Node *node;
-	PassageMarkerI marker, next;
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-
-		for (marker = getMarker(node); marker != NULL_IDX;
-		     marker = next) {
-			if (getPassageMarkerSequenceID(marker) > 0
-			    && isInitial(marker)
-			    && getReadLength(marker) < 400) {
-				destroyRead(marker);
-				next = getMarker(node);
-			} else if (getPassageMarkerSequenceID(marker) < 0
-				   && isTerminal(marker)
-				   && getReadLength(getTwinMarker(marker))
-				   < 400) {
-				destroyRead(getTwinMarker(marker));
-				next = getMarker(node);
-			} else
-				next = getNextInNode(marker);
-
-		}
-	}
-}
-
-Coordinate totalGraphLength(Graph * graph)
-{
-	IDnum index;
-	Coordinate totalLength = 0;
-
-	for (index = 1; index <= nodeCount(graph); index++)
-		totalLength += getNodeLength(getNodeInGraph(graph, index));
-
-	return totalLength;
-}
-
-void destroySinglePoolNodesStrict(Graph * graph)
-{
-	IDnum index;
-	Node *node;
-
-	velvetLog("Destroying single pool nodes\n");
-	resetNodeStatus(graph);
-
-	// Remove empty, single pool nodes, mark other single pool nodes
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-		if (getVirtualCoverage(node, 0) != 0
-		    && getVirtualCoverage(node, 1) != 0)
-			continue;
-
-		destroyNode(node, graph);
-	}
-
-	velvetLog("Done\n");
-
-	concatenateGraph(graph);
-}
-
-void contigStats(Node ** contigs, IDnum readCount)
-{
-	FibHeap *heap = newFibHeap();
-	IDnum index;
-	Coordinate totalLength = 0;
-	Coordinate sumLength = 0;
-	Node *node;
-	Coordinate halfLength;
-
-	for (index = 0; index <= readCount; index++) {
-		if (contigs[index] != NULL) {
-			node = contigs[index];
-			velvetLog("CONTIG %lld\n", (long long) getNodeLength(node));
-			insertNodeIntoHeap(heap, getNodeLength(node),
-					   node);
-			totalLength += getNodeLength(node);
-		}
-	}
-	halfLength = totalLength / 2;
-
-	node = removeNextNodeFromHeap(heap);
-	while (node != NULL) {
-		sumLength += getNodeLength(node);
-		if (sumLength >= halfLength)
-			break;
-		node = removeNextNodeFromHeap(heap);
-	}
-
-	destroyHeap(heap);
-	velvetLog("N50 %lld Total %lld\n", (long long) getNodeLength(node), (long long) totalLength);
-}
-
-void exportContigs(Node ** contigs, ReadSet * reads, char *filename,
-		   int WORDLENGTH, int pairedReadsCount)
-{
-	TightString **sequences =
-	    mallocOrExit(reads->readCount, TightString *);
-	IDnum i;
-
-	for (i = 0; i < pairedReadsCount; i++) {
-		if (contigs[i] == NULL)
-			sequences[i] = NULL;
-		else
-			sequences[i] = expandNode(contigs[i], WORDLENGTH);
-	}
-
-	exportSequenceArray(filename, sequences, reads->readCount);
-}
-
 static boolean terminalReferenceMarker(Node * node, ReadSet * reads) {
 	PassageMarkerI marker;
 
@@ -2270,6 +1134,39 @@ static Coordinate getLongCoverage(Node * node) {
 	return total;
 }
 
+void removeLowCoverageReferenceNodes(Graph * graph, double minCov, double minLongCov, ReadSet * reads)
+{
+	IDnum index;
+	Node *node;
+	PassageMarkerI marker;
+
+	velvetLog("Removing reference contigs with coverage < %f...\n", minCov);
+
+	for (index = 1; index <= nodeCount(graph); index++) {
+		node = getNodeInGraph(graph, index);
+
+		if (getNodeLength(node) == 0)
+			continue;
+
+		if ((getTotalCoverage(node) / getNodeLength(node) < minCov 
+		    || getLongCoverage(node) / getNodeLength(node) < minLongCov)
+		    && hasReferenceMarker(node, reads)) {
+			while ((marker = getMarker(node))) {
+				if (!isInitial(marker)
+				    && !isTerminal(marker))
+					disconnectNextPassageMarker
+					    (getPreviousInSequence(marker),
+					     graph);
+				destroyPassageMarker(marker);
+			}
+
+			destroyNode(node, graph);
+		}
+	}
+
+	concatenateGraph(graph);
+}
+
 void removeLowLongCoverageNodesAndDenounceDubiousReads(Graph * graph,
 						       double minCov,
 						       ReadSet * reads,
@@ -2422,38 +1319,6 @@ void removeLowLongCoverageNodesAndDenounceDubiousReads(Graph * graph,
 	concatenateGraph(graph);
 }
 
-void removeLowCoverageNodes(Graph * graph, double minCov)
-{
-	IDnum index;
-	Node *node;
-	PassageMarkerI marker;
-
-	if (minCov < 0)
-		return;
-
-	velvetLog("Applying a coverage cutoff of %f...\n", minCov);
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-
-		if (getNodeLength(node) > 0
-		    && getTotalCoverage(node) / getNodeLength(node) <
-		    minCov) {
-			while ((marker = getMarker(node)) != NULL_IDX) {
-				if (!isInitial(marker)
-				    && !isTerminal(marker))
-					disconnectNextPassageMarker
-					    (getPreviousInSequence(marker),
-					     graph);
-				destroyPassageMarker(marker);
-			}
-			destroyNode(node, graph);
-		}
-	}
-
-	concatenateGraph(graph);
-}
-
 void removeHighCoverageNodes(Graph * graph, double maxCov, boolean export, Coordinate minLength, char *filename)
 {
 	IDnum index;
@@ -2505,24 +1370,6 @@ void removeHighCoverageNodes(Graph * graph, double maxCov, boolean export, Coord
 	concatenateGraph(graph);
 }
 
-void removeMissingStrain(Graph * graph, Category cat)
-{
-	IDnum ID;
-	Node *node;
-
-	for (ID = 1; ID <= nodeCount(graph); ID++) {
-		node = getNodeInGraph(graph, ID);
-
-		if (node == NULL)
-			continue;
-
-		if (getVirtualCoverage(node, cat) == 0)
-			destroyNode(node, graph);
-	}
-
-	concatenateGraph(graph);
-}
-
 static void exportAMOSLib(FILE * outfile, Graph * graph, Category cat)
 {
 	Coordinate distance = getInsertLength(graph, cat * 2);
@@ -2856,134 +1703,6 @@ void exportAMOSContigs(char *filename, Graph * graph,
 
 }
 
-boolean isNatural(Graph * graph)
-{
-	Node *node;
-	IDnum index;
-
-	for (index = 1; index < nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-		if (node == NULL)
-			continue;
-
-		if (getNodeLength(node) == 0)
-			return false;
-
-		if (simpleArcCount(node) > 4)
-			return false;
-
-		if (simpleArcCount(getTwinNode(node)) > 4)
-			return false;
-	}
-
-	return true;
-}
-
-static Node *followShortPath(Arc * arc)
-{
-	Node *node = getDestination(arc);
-	if (simpleArcCount(node) != 1)
-		return NULL;
-	node = getDestination(getArc(node));
-	return getTwinNode(node);
-
-}
-
-static void checkNodeForHallidayJunction(Node * node, Graph * graph)
-{
-	Node *nodeA, *nodeB, *nodeC, *nodeD;
-	Arc *arc1, *arc2;
-
-	if (simpleArcCount(node) != 2)
-		return;
-
-	arc1 = getArc(node);
-	arc2 = getNextArc(arc1);
-
-	nodeA = followShortPath(arc1);
-	if (nodeA == NULL || simpleArcCount(nodeA) != 2
-	    || !isUniqueBasic(nodeA))
-		return;
-
-	nodeB = followShortPath(arc2);
-	if (nodeB == NULL || simpleArcCount(nodeB) != 2
-	    || !isUniqueBasic(nodeB))
-		return;
-
-	if (nodeA == nodeB) {
-		return;
-	}
-
-	arc1 = getArc(nodeA);
-	arc2 = getNextArc(arc1);
-	nodeC = followShortPath(arc1);
-	if (nodeC == NULL)
-		return;
-	if (nodeC == node) {
-		nodeC = followShortPath(arc2);
-		if (nodeC == NULL || nodeC == node
-		    || simpleArcCount(nodeC) != 2
-		    || !isUniqueBasic(nodeC)) {
-			velvetLog("NO %li %li %li %li\n", (long) getNodeID(node),
-			       (long) getNodeID(nodeA), (long) getNodeID(nodeB),
-			       (long) getNodeID(nodeC));
-			return;
-		}
-	} else {
-		if (simpleArcCount(nodeC) != 2 || !isUniqueBasic(nodeC)) {
-			velvetLog("2\n");
-			return;
-		}
-		nodeD = followShortPath(arc2);
-		if (nodeD != node) {
-			velvetLog("3\n");
-			return;
-		}
-	}
-
-	velvetLog("A\n");
-
-	arc1 = getArc(nodeB);
-	arc2 = getNextArc(arc1);
-	nodeD = followShortPath(arc1);
-	if (nodeD != nodeC && nodeD != node)
-		return;
-	nodeD = followShortPath(arc2);
-	if (nodeD != nodeC && nodeD != node)
-		return;
-
-	arc1 = getArc(nodeB);
-	arc2 = getNextArc(arc1);
-	nodeD = followShortPath(arc1);
-	if (nodeD != nodeC && nodeD != node)
-		return;
-	nodeD = followShortPath(arc2);
-	if (nodeD != nodeC && nodeD != node)
-		return;
-
-	velvetLog("JOHNNY HALLIDAY JUNCTION %li %li %li %li\n",
-	       (long) getNodeID(node), (long) getNodeID(nodeC), (long) getNodeID(nodeA),
-	       (long) getNodeID(nodeB));
-}
-
-void searchForHallidayJunction(Graph * graph)
-{
-	IDnum index;
-	Node *node;
-
-	setBaseCoverage(8);
-
-	for (index = 1; index <= nodeCount(graph); index++) {
-		node = getNodeInGraph(graph, index);
-
-		if (isUniqueBasic(node)) {
-			checkNodeForHallidayJunction(node, graph);
-			checkNodeForHallidayJunction(getTwinNode(node),
-						     graph);
-		}
-	}
-}
-
 Coordinate totalAssemblyLength(Graph * graph)
 {
 	IDnum index;
@@ -3168,7 +1887,7 @@ struct referenceCoord_st {
 static ReferenceCoord * collectReferenceCoords(char * sequencesFilename, IDnum referenceCount) {
 	FILE * file = fopen(sequencesFilename, "r");
 	char line[MAXLINE];
-	char name[500];
+	char name[5000];
 	Coordinate start, finish;
 	long long longlongvar;
 	IDnum refIndex = 0;
@@ -3201,8 +1920,8 @@ static ReferenceCoord * collectReferenceCoords(char * sequencesFilename, IDnum r
 					line[i] = '\0';
 				}
 
-				refCoords[refIndex].name = callocOrExit(strlen(name) + 1, char);  
 				strcpy(name, line + 1);
+				refCoords[refIndex].name = callocOrExit(strlen(name) + 1, char);  
 				strcpy(refCoords[refIndex].name, name);
 				refCoords[refIndex].start = 1;
 				refCoords[refIndex].finish = -1;
