@@ -20,6 +20,9 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 */
 #include <stdlib.h>
 #include <stdio.h>
+#ifdef OPENMP
+#include <omp.h>
+#endif
 
 #include "globals.h"
 #include "allocArray.h"
@@ -287,37 +290,6 @@ char *readPassageMarker(PassageMarkerI marker)
 	return s;
 }
 
-char *readPassageMarkerSequence(PassageMarkerI marker,
-				TightString ** sequences, int WORDLENGTH)
-{
-	TightString *sequence =
-	    sequences[getAbsolutePassMarkerSeqID(marker) - 1];
-	int i;
-	char *s = NULL;
-
-	if (marker == NULL_IDX)
-		return s;
-
-	s = mallocOrExit(getPassageMarkerLength(marker) + 1, char);
-
-	if (getPassageMarkerSequenceID(marker) > 0)
-		for (i = 0; i < getPassageMarkerLength(marker); i++)
-			s[i] =
-			    getNucleotideChar(getPassageMarkerStart(marker)
-					      + i + WORDLENGTH - 1,
-					      sequence);
-	else
-		for (i = 0; i < getPassageMarkerLength(marker); i++)
-			s[i] =
-			    getInverseNucleotideChar(getPassageMarkerStart
-						     (marker) - i - 1,
-						     sequence);
-
-	s[getPassageMarkerLength(marker)] = '\0';
-
-	return s;
-}
-
 PassageMarkerI addPassageMarker(IDnum sequenceID, Coordinate start,
 				Node * node)
 {
@@ -370,39 +342,6 @@ PassageMarkerList *copyPassageMarkerList(PassageMarkerList * list)
 	return result;
 }
 
-PassageMarkerI copyPassageMarker(PassageMarkerI marker)
-{
-	PassageMarker *markerVal = PM_FI2P (marker);
-	PassageMarker *twin = PM_FI2P (markerVal->twinMarker);
-	PassageMarkerI copy = allocatePassageMarker();
-	PassageMarkerI twinCopy = allocatePassageMarker();
-	PassageMarker *copyVal = PM_FI2P (copy);
-	PassageMarker *twinCopyVal = PM_FI2P (twinCopy);
-
-	copyVal->sequenceID = markerVal->sequenceID;
-	copyVal->start = markerVal->start;
-	copyVal->nextInNode = NULL_IDX;
-	copyVal->previousInNode = NULL_IDX;
-	copyVal->node = NULL;
-	copyVal->nextInSequence = markerVal->nextInSequence;
-	copyVal->finishOffset = markerVal->finishOffset;
-	copyVal->status = false;
-
-	twinCopyVal->sequenceID = twin->sequenceID;
-	twinCopyVal->start = twin->start;
-	twinCopyVal->nextInNode = NULL_IDX;
-	twinCopyVal->previousInNode = NULL_IDX;
-	twinCopyVal->node = NULL;
-	twinCopyVal->nextInSequence = twin->nextInSequence;
-	twinCopyVal->finishOffset = twin->finishOffset;
-	twinCopyVal->status = false;
-
-	copyVal->twinMarker = twinCopy;
-	twinCopyVal->twinMarker = copy;
-
-	return copy;
-}
-
 void incrementFinishOffset(PassageMarkerI marker, Coordinate offset)
 {
 	PM_FI2P (marker)->finishOffset += offset;
@@ -647,10 +586,22 @@ PassageMarkerI newPassageMarker(IDnum seqID, Coordinate start,
 				Coordinate finish, Coordinate startOffset,
 				Coordinate finishOffset)
 {
-	PassageMarkerI marker = allocatePassageMarker();
-	PassageMarkerI twinMarker = allocatePassageMarker();
-	PassageMarker *markerVal = PM_FI2P (marker);
-	PassageMarker *twinMarkerVal = PM_FI2P (twinMarker);
+	PassageMarkerI marker;
+	PassageMarkerI twinMarker;
+	PassageMarker *markerVal;
+	PassageMarker *twinMarkerVal;
+
+#ifdef OPENMP
+	#pragma omp critical
+	{
+#endif
+		marker = allocatePassageMarker();
+		twinMarker = allocatePassageMarker();
+#ifdef OPENMP
+	}
+#endif
+	markerVal = PM_FI2P (marker);
+	twinMarkerVal = PM_FI2P (twinMarker);
 
 //      velvetLog("Values %d\t%d\t%d\t%d\t%d\n", seqID, start, finish, startOffset, finishOffset);
 
