@@ -74,35 +74,17 @@ static void deallocateRBConnection(RBConnection * nodeList)
 	deallocatePointer(nodeListMemory, nodeList);
 }
 
-void setBaseCoverage(double coverage)
-{
-	expected_coverage = coverage;
-}
-
-boolean isUniqueBasic(Node * node)
-{
-	if (getNodeLength(node) < LONG_NODE_CUTOFF) {
-		return false;
-	}
-	if (readCoverage(node) / (double) getNodeLength(node) >
-	    1.5 * expected_coverage) {
-		return false;
-	}
-
-	return true;
-}
-
 boolean isUniqueSolexa(Node * node)
 {
 
 	Coordinate nodeLength = getNodeLength(node);
-	Coordinate nodeCoverage =
-	    (getVirtualCoverage(node, 0) + getVirtualCoverage(node, 1));
+	Coordinate nodeCoverage = 0;
 	double nodeDensity, probability;
+	Category cat;
+
+	for (cat = 0; cat < CATEGORIES; cat++)
+		nodeCoverage += getVirtualCoverage(node, cat);
 
-	if (nodeLength == 0) {
-		return false;
-	}
 	if (nodeLength > LONG_NODE_CUTOFF) {
 		nodeDensity = nodeCoverage / (double) nodeLength;
 
@@ -112,12 +94,8 @@ boolean isUniqueSolexa(Node * node)
 		    (expected_coverage * expected_coverage -
 		     nodeDensity * nodeDensity / 2);
 		return probability > PROBABILITY_CUTOFF;
-	} else {
-		return false;
-		probability =
-		    expected_coverage * nodeLength - nodeCoverage / LN2;
-		return probability > 0;
 	}
+	return false;
 }
 
 static void identifyUniqueNodes(boolean(*isUniqueFunction) (Node *))
