@@ -151,7 +151,7 @@ static void addReferenceCoordinate(ReferenceCoordinateTable * table, char * name
 		velvetLog("Overlapping reference coordinates:\n");
 		velvetLog("%s:%lli-%lli\n", name, (long long) start, (long long) finish);
 		velvetLog("%s:%lli-%lli\n", refCoord->name, (long long) refCoord->start, (long long) refCoord->finish);
-		velvetLog("Exiting...");
+		velvetLog("Exiting...\n");
 #ifdef DEBUG 
 		abort();
 #endif 
@@ -248,215 +248,6 @@ static void writeFastaSequence(FILE * outfile, const char * str)
 		velvetFprintf(outfile, "%.60s\n", &str[start]);
 }
 
-ReadSet *newReadSetAroundTightStringArray(TightString ** array,
-					  IDnum length)
-{
-	ReadSet *rs = newReadSet();
-	/*
-	rs->tSequences = array;
-	rs->readCount = length;
-	*/
-	return rs;
-}
-
-void concatenateReadSets(ReadSet * A, ReadSet * B)
-{
-/*
-	ReadSet tmp;
-	IDnum index;
-
-	// Read count:  
-	tmp.readCount = A->readCount + B->readCount;
-
-	// Sequences
-	if (A->sequences != NULL || B->sequences != NULL) {
-		tmp.sequences = mallocOrExit(tmp.readCount, char *);
-		if (A->sequences != NULL) {
-			for (index = 0; index < A->readCount; index++)
-				tmp.sequences[index] = A->sequences[index];
-			free(A->sequences);
-		} else
-			for (index = 0; index < A->readCount; index++)
-				tmp.sequences[index] = NULL;
-
-		if (B->sequences != NULL) {
-			for (index = 0; index < B->readCount; index++)
-				tmp.sequences[A->readCount + index] =
-				    B->sequences[index];
-			free(B->sequences);
-		} else
-			for (index = 0; index < B->readCount; index++)
-				tmp.sequences[A->readCount + index] = NULL;
-	} else
-		tmp.sequences = NULL;
-
-	// tSequences
-	if (A->tSequences != NULL || B->tSequences != NULL) {
-		tmp.tSequences =
-		    mallocOrExit(tmp.readCount, TightString *);
-
-		if (A->tSequences != NULL) {
-			for (index = 0; index < A->readCount; index++)
-				tmp.tSequences[index] =
-				    A->tSequences[index];
-			free(A->tSequences);
-		} else
-			for (index = 0; index < A->readCount; index++)
-				tmp.tSequences[index] = NULL;
-
-		if (B->tSequences != NULL) {
-			for (index = 0; index < B->readCount; index++)
-				tmp.tSequences[A->readCount + index] =
-				    B->tSequences[index];
-			free(B->tSequences);
-		} else
-			for (index = 0; index < B->readCount; index++)
-				tmp.tSequences[A->readCount + index] =
-				    NULL;
-	} else
-		tmp.tSequences = NULL;
-
-	// Labels
-	if (A->labels != NULL || B->labels != NULL) {
-		tmp.labels = mallocOrExit(tmp.readCount, char *);
-
-		if (A->labels != NULL) {
-			for (index = 0; index < A->readCount; index++)
-				tmp.labels[index] = A->labels[index];
-			free(A->labels);
-		} else
-			for (index = 0; index < A->readCount; index++)
-				tmp.labels[index] = NULL;
-
-		if (B->labels != NULL) {
-			for (index = 0; index < B->readCount; index++)
-				tmp.labels[A->readCount + index] =
-				    B->labels[index];
-			free(B->labels);
-		} else
-			for (index = 0; index < B->readCount; index++)
-				tmp.labels[A->readCount + index] = NULL;
-	} else
-		tmp.labels = NULL;
-
-
-	// Confidence scores
-	if (A->confidenceScores != NULL || B->confidenceScores != NULL) {
-		tmp.confidenceScores =
-		    mallocOrExit(tmp.readCount, Quality *);
-
-		if (A->confidenceScores != NULL) {
-			for (index = 0; index < A->readCount; index++)
-				tmp.confidenceScores[index] =
-				    A->confidenceScores[index];
-			free(A->confidenceScores);
-		} else
-			for (index = 0; index < A->readCount; index++)
-				tmp.confidenceScores[index] = NULL;
-
-		if (B->confidenceScores != NULL) {
-			for (index = 0; index < B->readCount; index++)
-				tmp.confidenceScores[A->readCount +
-						     index] =
-				    B->confidenceScores[index];
-			free(B->confidenceScores);
-		} else
-			for (index = 0; index < B->readCount; index++)
-				tmp.confidenceScores[A->readCount +
-						     index] = NULL;
-	} else
-		tmp.confidenceScores = NULL;
-
-	// Kmer probabilities 
-	if (A->kmerProbabilities != NULL || B->kmerProbabilities != NULL) {
-		tmp.kmerProbabilities =
-		    mallocOrExit(tmp.readCount, Quality *);
-
-		if (A->kmerProbabilities != NULL) {
-			for (index = 0; index < A->readCount; index++)
-				tmp.kmerProbabilities[index] =
-				    A->kmerProbabilities[index];
-			free(A->kmerProbabilities);
-		} else
-			for (index = 0; index < A->readCount; index++)
-				tmp.kmerProbabilities[index] = NULL;
-
-		if (B->kmerProbabilities != NULL) {
-			for (index = 0; index < B->readCount; index++)
-				tmp.kmerProbabilities[A->readCount +
-						      index] =
-				    B->kmerProbabilities[index];
-			free(B->kmerProbabilities);
-		} else
-			for (index = 0; index < B->readCount; index++)
-				tmp.kmerProbabilities[A->readCount +
-						      index] = NULL;
-	} else
-		tmp.kmerProbabilities = NULL;
-
-	// Mate reads 
-	if (A->mateReads != NULL || B->mateReads != NULL) {
-		tmp.mateReads = mallocOrExit(tmp.readCount, IDnum);
-
-		if (A->mateReads != NULL) {
-			for (index = 0; index < A->readCount; index++)
-				tmp.mateReads[index] = A->mateReads[index];
-			free(A->mateReads);
-		} else
-			for (index = 0; index < A->readCount; index++)
-				tmp.mateReads[index] = 0;
-
-		if (B->mateReads != NULL) {
-			for (index = 0; index < B->readCount; index++)
-				tmp.mateReads[A->readCount + index] =
-				    B->mateReads[index];
-			free(B->mateReads);
-		} else
-			for (index = 0; index < B->readCount; index++)
-				tmp.mateReads[A->readCount + index] = 0;
-	} else
-		tmp.mateReads = NULL;
-
-	// Categories
-	if (A->categories != NULL || B->categories != NULL) {
-		tmp.categories = mallocOrExit(tmp.readCount, Quality *);
-
-		if (A->categories != NULL) {
-			for (index = 0; index < A->readCount; index++)
-				tmp.categories[index] =
-				    A->categories[index];
-			free(A->categories);
-		} else
-			for (index = 0; index < A->readCount; index++)
-				tmp.categories[index] = CATEGORIES;
-
-		if (B->categories != NULL) {
-			for (index = 0; index < B->readCount; index++)
-				tmp.categories[A->readCount + index] =
-				    B->categories[index];
-			free(B->categories);
-		} else
-			for (index = 0; index < B->readCount; index++)
-				tmp.categories[A->readCount + index] =
-				    CATEGORIES;
-	} else
-		tmp.categories = NULL;
-
-	// Put everything back into A
-	A->readCount = tmp.readCount;
-	A->sequences = tmp.sequences;
-	A->tSequences = tmp.tSequences;
-	A->labels = tmp.labels;
-	A->confidenceScores = tmp.confidenceScores;
-	A->kmerProbabilities = tmp.kmerProbabilities;
-	A->mateReads = tmp.mateReads;
-	A->categories = tmp.categories;
-
-	// Deallocate
-	free(B);
-*/
-}
-
 void convertSequences(ReadSet * rs)
 {
 	rs->tSequences = newTightStringArrayFromStringArray(rs->sequences,
@@ -465,107 +256,6 @@ void convertSequences(ReadSet * rs)
 	rs->sequences = NULL;
 }
 
-static Probability convertQualityScore(Quality score)
-{
-	return (Probability) 1 - pow(10, -score / ((double) 10));
-}
-
-void convertConfidenceScores(ReadSet * rs, int WORDLENGTH)
-{
-	Quality *baseCallerScores;
-	Probability *kmerProbabilities;
-	IDnum index;
-	Coordinate position;
-	Probability proba;
-
-	rs->kmerProbabilities =
-	    mallocOrExit(rs->readCount, Probability *);
-
-	for (index = 0; index < rs->readCount; index++) {
-		rs->kmerProbabilities[index] =
-		    mallocOrExit(getLength(getTightStringInArray(rs->tSequences, index)) - WORDLENGTH +
-			    1, Probability);
-		kmerProbabilities = rs->kmerProbabilities[index];
-		baseCallerScores = rs->confidenceScores[index];
-
-		proba = 1;
-		for (position = 0;
-		     position < getLength(getTightStringInArray(rs->tSequences, index));
-		     position++) {
-			proba *=
-			    convertQualityScore(baseCallerScores
-						[position]);
-			if (position < WORDLENGTH)
-				continue;
-
-			proba /=
-			    convertQualityScore(baseCallerScores
-						[position - WORDLENGTH]);
-			kmerProbabilities[position - WORDLENGTH + 1] =
-			    proba;
-		}
-
-		rs->confidenceScores[index] = NULL;
-		free(baseCallerScores);
-	}
-
-	free(rs->confidenceScores);
-	rs->confidenceScores = NULL;
-}
-
-void categorizeReads(ReadSet * readSet, Category category)
-{
-	IDnum index;
-
-	if (readSet->categories == NULL) 
-		readSet->categories =
-		    mallocOrExit(readSet->readCount, Category);
-
-	for (index = 0; index < readSet->readCount; index++)
-		readSet->categories[index] = category;
-}
-
-void simplifyReads(ReadSet * readSet)
-{
-	IDnum index;
-
-	if (readSet->categories == NULL)
-		readSet->categories =
-		    mallocOrExit(readSet->readCount, Category);
-
-	for (index = 0; index < readSet->readCount; index++) {
-		if (readSet->categories[index] < CATEGORIES) {
-			readSet->categories[index] /= 2;
-			readSet->categories[index] *= 2;
-		}
-	}
-}
-
-void exportIDMapping(char *filename, ReadSet * reads)
-{
-	IDnum index;
-	FILE *outfile = fopen(filename, "w");
-
-	if (outfile == NULL) {
-		velvetLog("Couldn't open %s, sorry\n", filename);
-		return;
-	} else
-		velvetLog("Writing into file...\n");
-
-	if (reads->labels == NULL) {
-		fclose(outfile);
-		return;
-	}
-
-	for (index = 0; index < reads->readCount; index++)
-		if (reads->labels != NULL)
-			velvetFprintf(outfile, "s/SEQUENCE %ld/%s/\n", (long) (index + 1),
-				reads->labels[index]);
-
-	fclose(outfile);
-
-}
-
 // Returns the value of a 32-bit little-endian-stored integer.
 static int int32(const unsigned char * ptr)
 {
@@ -1565,15 +1255,16 @@ static void printUsage()
 
 // General argument parser for most functions
 // Basically a reused portion of toplevel code dumped into here
-void parseDataAndReadFiles(char * filename, int argc, char **argv, boolean * double_strand)
+void parseDataAndReadFiles(char * filename, int argc, char **argv, boolean * double_strand, boolean * noHash)
 {
 	int argIndex = 1;
-	FILE *outfile = fopen(filename, "w");
+	FILE *outfile;
 	int filetype = FASTA;
 	Category cat = 0;
 	IDnum sequenceIndex = 1;
 	short short_var;
 	ReferenceCoordinateTable * refCoords = newReferenceCoordinateTable();
+	boolean reuseSequences = false;
 
 	if (argc < 2) {
 		printUsage();
@@ -1583,6 +1274,22 @@ void parseDataAndReadFiles(char * filename, int argc, char **argv, boolean * dou
 		exit(1);
 	}
 
+	for (argIndex = 1; argIndex < argc; argIndex++) {
+		if (strcmp(argv[argIndex], "-strand_specific") == 0) {
+			*double_strand = false;
+			reference_coordinate_double_strand = false;
+		} else if (strcmp(argv[argIndex], "-reuse_Sequences") == 0) {
+			reuseSequences = true;
+		} else if (strcmp(argv[argIndex], "-noHash") == 0) {
+			*noHash = true;
+		}
+	}
+
+	if (reuseSequences) 
+		return;
+
+	outfile = fopen(filename, "w");
+
 	for (argIndex = 1; argIndex < argc; argIndex++) {
 		if (argv[argIndex][0] == '-' && strlen(argv[argIndex]) > 1) {
 
@@ -1651,19 +1358,11 @@ void parseDataAndReadFiles(char * filename, int argc, char **argv, boolean * dou
 			else if (strcmp(argv[argIndex], "-reference") ==
 				 0)
 				cat = CATEGORIES * 2 + 2;
-			else if (strcmp(argv[argIndex], "-strand_specific")
-				 == 0) {
-				if (argIndex != 1) {
-					velvetLog("Error: if using the -strand_specific flag, it must be placed right after the hash length\n");
-					velvetLog("Sorry for the inconvenience\n");
-#ifdef DEBUG 
-					abort();
-#endif 
-					exit(1);
-				}
+			else if (strcmp(argv[argIndex], "-strand_specific") == 0) {
 				*double_strand = false;
 				reference_coordinate_double_strand = false;
-			} else {
+			} 
+			else {
 				velvetLog("Unknown option: %s\n",
 				       argv[argIndex]);
 #ifdef DEBUG 
@@ -1725,41 +1424,124 @@ void parseDataAndReadFiles(char * filename, int argc, char **argv, boolean * dou
 	fclose(outfile);
 }
 
-void createReadPairingArray(ReadSet* reads) {
+void createReadPairingArray(ReadSet* reads)
+{
 	IDnum index;
 	IDnum *mateReads = mallocOrExit(reads->readCount, IDnum);
+	Category cat = 0;
+	int phase = 0;
 
 	for (index = 0; index < reads->readCount; index++) 
 		mateReads[index] = -1;
 
 	reads->mateReads = mateReads;
-}
 
-boolean pairUpReads(ReadSet * reads, Category cat)
-{
-	int phase = 0;
-	IDnum index;
-	boolean found = false;
-
-	for (index = 0; index < reads->readCount; index++) {
-		if (reads->categories[index] != cat) {
-			if (phase == 1) {
-				reads->mateReads[index - 1] = -1;
-				reads->categories[index - 1]--;
+	for (index = 0; index < reads->readCount; index++)
+	{
+		// Paired category
+		if (cat & 1)
+		{
+			// Leaving the paired category
+			if (reads->categories[index] != cat)
+			{
+				if (phase == 1)
+				{
+					reads->mateReads[index - 1] = -1;
+					reads->categories[index - 1]--;
+					phase = 0;
+				}
+				cat = reads->categories[index];
+				// Into another paired category
+				if (cat & 1)
+				{
+					reads->mateReads[index] = index + 1;
+					phase = 1;
+				}
+			}
+			else if (phase == 0)
+			{
+				reads->mateReads[index] = index + 1;
+				phase = 1;
+			}
+			else
+			{
+				reads->mateReads[index] = index - 1;
 				phase = 0;
 			}
-		} else if (phase == 0) {
-			found = true;
-			reads->mateReads[index] = index + 1;
-			phase = 1;
-		} else {
-			found = true;
-			reads->mateReads[index] = index - 1;
-			phase = 0;
+		}
+		// Leaving an unpaired category
+		else if (reads->categories[index] != cat)
+		{
+			cat = reads->categories[index];
+			// Into a paired category
+			if (cat & 1)
+			{
+				reads->mateReads[index] = index + 1;
+				phase = 1;
+			}
 		}
 	}
+}
 
-	return found;
+int pairedCategories(ReadSet * reads)
+{
+	boolean pairedCat[CATEGORIES + 1];
+	int pairedCatCount = 0;
+	IDnum index;
+
+	for (index = 0; index <= CATEGORIES; index++)
+		pairedCat[index] = 0;
+
+	for (index = 0; index < reads->readCount; index++) {
+		if (reads->categories[index] & 1 && !pairedCat[reads->categories[index] / 2]) {
+			pairedCat[reads->categories[index] / 2] = true;
+			if (pairedCatCount++ == CATEGORIES)
+				break;
+		} 	
+	}
+
+	return pairedCatCount;
+}
+
+boolean isSecondInPair(ReadSet * reads, IDnum index)
+{
+	return reads->secondInPair[index / 8] & (1 << (index & 7));
+}
+
+
+static void computeSecondInPair(ReadSet * reads)
+{
+	IDnum index;
+	Category currentCat = 0;
+	Category previousCat = 0;
+	int phase = 0;
+
+	if (reads->secondInPair)
+		free (reads->secondInPair);
+	reads->secondInPair = callocOrExit((reads->readCount + 7) / 8, char);
+
+	for (index = 0; index < reads->readCount; index++)
+	{
+		currentCat = reads->categories[index];
+		if (currentCat & 1)
+		{
+			if (previousCat == currentCat)
+			{
+				if (phase == 0)
+				{
+					phase = 1;
+				}
+				else
+				{
+					reads->secondInPair[index / 8] |= (1 << (index & 7));
+					phase = 0;
+				}
+			}
+			else
+				phase = 0;
+		}
+		previousCat = currentCat;
+	}
 }
 
 void detachDubiousReads(ReadSet * reads, boolean * dubiousReads)
@@ -1773,67 +1555,19 @@ void detachDubiousReads(ReadSet * reads, boolean * dubiousReads)
 		return;
 
 	for (index = 0; index < sequenceCount; index++) {
-		if (!dubiousReads[index])
+		if (!dubiousReads[index] || reads->categories[index] % 2 == 0 )
 			continue;
 
-		pairID = mateReads[index];
+		if (isSecondInPair(reads, index))
+		    pairID = index - 1;
+		else
+		    pairID = index + 1;
 
-		if (pairID != -1) {
-			//velvetLog("Separating %d and %d\n", index, pairID);
-			mateReads[index] = -1;
-			mateReads[pairID] = -1;
-		}
+		reads->categories[index] = (reads->categories[index] / 2) * 2;
+		reads->categories[pairID] = (reads->categories[pairID] / 2) * 2;
 	}
 }
 
-static void exportRead(FILE * outfile, ReadSet * reads, IDnum index)
-{
-	Coordinate start, finish;
-	char str[100];
-	TightString *sequence = getTightStringInArray(reads->tSequences, index);
-
-	if (sequence == NULL)
-		return;
-
-	velvetFprintf(outfile, ">SEQUENCE_%ld_length_%lld", (long) index,
-		(long long) getLength(sequence));
-
-	if (reads->categories != NULL)
-		velvetFprintf(outfile, "\t%i", (int) reads->categories[index]);
-
-	velvetFprintf(outfile, "\n");
-
-	start = 0;
-	while (start <= getLength(sequence)) {
-		finish = start + 60;
-		readTightStringFragment(sequence, start, finish, str);
-		velvetFprintf(outfile, "%s\n", str);
-		start = finish;
-	}
-
-	fflush(outfile);
-}
-
-void exportReadSet(char *filename, ReadSet * reads)
-{
-	IDnum index;
-	FILE *outfile = fopen(filename, "w+");
-
-	if (outfile == NULL) {
-		velvetLog("Couldn't open file, sorry\n");
-		return;
-	} else
-		velvetLog("Writing into readset file: %s\n", filename);
-
-	for (index = 0; index < reads->readCount; index++) {
-		exportRead(outfile, reads, index);
-	}
-
-	fclose(outfile);
-
-	velvetLog("Done\n");
-}
-
 ReadSet *importReadSet(char *filename)
 {
 	FILE *file = fopen(filename, "r");
@@ -1842,9 +1576,9 @@ ReadSet *importReadSet(char *filename)
 	const int maxline = 5000;
 	char line[5000];
 	IDnum sequenceCount, sequenceIndex;
-	IDnum index;
 	ReadSet *reads;
 	short int temp_short;
+	int lineLength;
 
 	if (file != NULL)
 		velvetLog("Reading read set file %s;\n", filename);
@@ -1895,7 +1629,7 @@ ReadSet *importReadSet(char *filename)
 			if (sizeof(ShortLength) == sizeof(int16_t) && bpCount > SHRT_MAX) {
 				velvetLog("Read %li of length %lli, longer than limit %i\n",
 				       (long) sequenceIndex + 1, (long long) bpCount, SHRT_MAX);
-				velvetLog("You should modify ShortLength to int32_t in globals.h (around) line 84, recompile and re-run\n");
+				velvetLog("You should modify recompile with the LONGSEQUENCES option (cf. manual)\n");
 				exit(1);
 			}
 		}
@@ -1922,15 +1656,15 @@ ReadSet *importReadSet(char *filename)
 		} else if (line[0] == 'M') {;
 			// Map line
 		} else {
-			for (index = 0; index < (Coordinate) strlen(line) - 1;
-			     index++)
-				sequence[bpCount + index] = line[index];
-			bpCount += (Coordinate) (strlen(line) - 1);
+			lineLength = strlen(line) - 1;
+			strncpy(sequence + bpCount, line, lineLength);
+			bpCount += (Coordinate) lineLength;
 		}
 	}
 
 	sequence[bpCount] = '\0';
 	fclose(file);
+	computeSecondInPair(reads);
 
 	velvetLog("Done\n");
 	return reads;
@@ -2004,6 +1738,7 @@ void destroyReadSet(ReadSet * reads)
 	free(reads->kmerProbabilities);
 	free(reads->mateReads);
 	free(reads->categories);
+	free(reads->secondInPair);
 	free(reads);
 }
 
@@ -2019,46 +1754,3 @@ IDnum *getSequenceLengths(ReadSet * reads, int wordLength)
 
 	return lengths;
 }
-
-IDnum *getSequenceLengthsFromFile(char *filename, int wordLength) /* SF TODO This could also be chopped down to IDnum */
-{
-	IDnum *lengths;
-	FILE *file = fopen(filename, "r");
-	Coordinate bpCount = 0;
-	const int maxline = 100;
-	char line[100];
-	IDnum sequenceCount, sequenceIndex;
-	int lengthOffset = wordLength - 1;
-
-	if (file != NULL)
-		velvetLog("Reading read set file %s;\n", filename);
-	else 
-		exitErrorf(EXIT_FAILURE, true, "Could not open %s", filename);
-
-	// Count number of separate sequences
-	sequenceCount = 0;
-	while (fgets(line, maxline, file) != NULL)
-		if (line[0] == '>')
-			sequenceCount++;
-	fclose(file);
-
-	lengths = callocOrExit(sequenceCount, IDnum);
-	// Counting base pair length of each sequence:
-	file = fopen(filename, "r");
-	sequenceIndex = -1;
-	while (fgets(line, maxline, file) != NULL) {
-		if (line[0] == '>') {
-			if (sequenceIndex != -1)
-				lengths[sequenceIndex] =
-				    bpCount - lengthOffset;
-			sequenceIndex++;
-			bpCount = 0;
-		} else {
-			bpCount += (Coordinate) strlen(line) - 1;
-		}
-	}
-	lengths[sequenceIndex] =  bpCount - lengthOffset;
-	fclose(file);
-
-	return lengths;
-}
