@@ -52,14 +52,16 @@ static void printUsage()
 	puts("\t-max_branch_length <integer>\t: maximum length in base pair of bubble (default: 100)");
 	puts("\t-max_divergence <floating-point>: maximum divergence rate between two branches in a bubble (default: 0.2)");
 	puts("\t-max_gap_count <integer>\t: maximum number of gaps allowed in the alignment of the two branches of a bubble (default: 3)");
-	puts("\t-min_pair_count <integer>\t: minimum number of paired end connections to justify the scaffolding of two long contigs (default: 10)");
+	puts("\t-min_pair_count <integer>\t: minimum number of paired end connections to justify the scaffolding of two long contigs (default: 5)");
 	puts("\t-max_coverage <floating point>\t: removal of high coverage nodes AFTER tour bus (default: no removal)");
 	puts("\t-long_mult_cutoff <int>\t\t: minimum number of long reads required to merge contigs (default: 2)");
 	puts("\t-unused_reads <yes|no>\t\t: export unused reads in UnusedReads.fa file (default: no)");
 	puts("\t-alignments <yes|no>\t\t: export a summary of contig alignment to the reference sequences (default: no)");
 	puts("\t-exportFiltered <yes|no>\t: export the long nodes which were eliminated by the coverage filters (default: no)");
-	puts("\t-clean <yes|no>\t\t: remove all the intermediary files which are useless for recalculation (default : no)");
+	puts("\t-clean <yes|no>\t\t\t: remove all the intermediary files which are useless for recalculation (default : no)");
 	puts("\t-very_clean <yes|no>\t\t: remove all the intermediary files (no recalculation possible) (default: no)");
+	puts("\t-paired_exp_fraction <double>\t: remove all the paired end connections which less than the specified fraction of the expected count (default: 0.1)");
+	puts("\t-shortMatePaired* <yes|no>\t: for mate-pair libraries, indicate that the library might be contaminated with paired-end reads (default no)");
 	puts("");
 	puts("Output:");
 	puts("\tdirectory/contigs.fa\t\t: fasta file of contigs longer than twice hash length");
@@ -80,7 +82,6 @@ int main(int argc, char **argv)
 	double longCoverageCutoff = -1;
 	double maxCoverageCutoff = -1;
 	double expectedCoverage = -1;
-	int longMultCutoff = -1;
 	Coordinate minContigLength = -1;
 	Coordinate minContigKmerLength;
 	boolean *dubious = NULL;
@@ -107,12 +108,14 @@ int main(int argc, char **argv)
 	short int short_var;
 	boolean exportFilteredNodes = false;
 	int clean = 0;
+	boolean shadows[CATEGORIES];
 
 	setProgramName("velvetg");
 
 	for (cat = 0; cat < CATEGORIES; cat++) {
 		insertLength[cat] = -1;
 		std_dev[cat] = -1;
+		shadows[cat] = false;
 	}
 
 	// Error message
@@ -288,8 +291,11 @@ int main(int argc, char **argv)
 		} else if (strcmp(arg, "-max_coverage") == 0) {
 			sscanf(argv[arg_index], "%lf", &maxCoverageCutoff);
 		} else if (strcmp(arg, "-long_mult_cutoff") == 0) {
-			sscanf(argv[arg_index], "%i", &longMultCutoff);
-			setMultiplicityCutoff(longMultCutoff);
+			sscanf(argv[arg_index], "%i", &arg_int);
+			setMultiplicityCutoff(arg_int);
+		} else if (strcmp(arg, "-paired_exp_fraction") == 0) {
+			sscanf(argv[arg_index], "%lf", &arg_double);
+			setPairedExpFraction(arg_double);
 		} else if (strcmp(arg, "-clean") == 0) {
 			if (strcmp(argv[arg_index], "yes") == 0)
 				clean = 1;
@@ -301,9 +307,22 @@ int main(int argc, char **argv)
 			    (strcmp(argv[arg_index], "yes") == 0);
 			if (unusedReads)
 				readTracking = true;
+		} else if (strcmp(arg, "-shortMatePaired") == 0) {
+			shadows[0] = (strcmp(argv[arg_index], "yes") == 0);
+		} else if (strncmp(arg, "-shortMatePaired", 16) == 0) {
+			sscanf(arg, "-shortMatePaired%hi", &short_var);
+			cat = (Category) short_var;
+			if (cat < 1 || cat > CATEGORIES) {
+				velvetLog("Unknown option: %s\n", arg);
+#ifdef DEBUG
+				abort();
+#endif
+				exit(1);
+			}
+			shadows[cat - 1] = (strcmp(argv[arg_index], "yes") == 0);
 		} else if (strcmp(arg, "--help") == 0) {
 			printUsage();
-			return 0;	
+			return 0;
 		} else {
 			velvetLog("Unknown option: %s;\n", arg);
 			printUsage();
@@ -448,19 +467,13 @@ int main(int argc, char **argv)
 		readCoherentGraph(graph, isUniqueSolexa, expectedCoverage,
 				  sequences);
 
-		// Paired ends module
+		// Paired end resolution
 		createReadPairingArray(sequences);
-		for (cat = 0; cat < CATEGORIES; cat++) 
-			if(pairUpReads(sequences, 2 * cat + 1))
-				pebbleRounds++;
-
-		if (pairUpReads(sequences, LONG_PAIRED))
-			pebbleRounds++;
-
+		pebbleRounds += pairedCategories(sequences);
 		detachDubiousReads(sequences, dubious);
 		activateGapMarkers(graph);
 		for ( ;pebbleRounds > 0; pebbleRounds--)
-			exploitShortReadPairs(graph, sequences, dubious, scaffolding);
+			exploitShortReadPairs(graph, sequences, dubious, shadows, scaffolding);
 	} else {
 		velvetLog("WARNING: NO EXPECTED COVERAGE PROVIDED\n");
 		velvetLog("Velvet will be unable to resolve any repeats\n");
@@ -471,6 +484,8 @@ int main(int argc, char **argv)
 
 	concatenateGraph(graph);
 
+	removeLowCoverageReferenceNodes(graph, coverageCutoff, longCoverageCutoff, sequences);
+
 	strcpy(graphFilename, directory);
 	strcat(graphFilename, "/contigs.fa");
 	exportLongNodeSequences(graphFilename, graph, minContigKmerLength); 
@@ -513,16 +528,16 @@ int main(int argc, char **argv)
 		strcat(graphFilename, "/Roadmaps");
 		remove(graphFilename);	
 
-		strcpy(graphFilename, directory);
-		strcat(graphFilename, "/Sequences");
-		remove(graphFilename);	
-
 		strcpy(graphFilename, directory);
 		strcat(graphFilename, "/LastGraph");
 		remove(graphFilename);	
 	} 
 
 	if (clean > 1) {
+		strcpy(graphFilename, directory);
+		strcat(graphFilename, "/Sequences");
+		remove(graphFilename);	
+
 		strcpy(graphFilename, directory);
 		strcat(graphFilename, "/Graph2.txt");
 		remove(graphFilename);	
