@@ -25,11 +25,7 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 
 typedef struct splayNode_st SplayTree;
 
-// Creates an empty tree (=NULL in fact)
-SplayTree *newSplayTree();
-
 // Deallocates tree memory
-void destroySplayTree(SplayTree * T);
 void destroyAllSplayTrees();
 
 // Finds occurrence of kmer in the tree
@@ -39,13 +35,9 @@ boolean findOrInsertOccurenceInSplayTree(Kmer * kmer, IDnum * seqID,
 					 Coordinate * position,
 					 SplayTree ** T);
 
-boolean placeOccurenceInSplayTree(Kmer * kmer, IDnum * seqID,
-				  Coordinate * position, SplayTree ** T);
+#ifdef OPENMP
+/* Initialises the per-thread RecycleBin array */
+void initSplayTreeMemory(void);
+#endif
 
-void countOccurenceInSplayTree(Kmer * kmer, SplayTree ** T, int increment);
-
-void filterAndExportSplayTree(FILE * file, SplayTree * T, int minCov,
-			      int maxCov);
-
-void displaySplayTreeMemory();
 #endif
