@@ -20,21 +20,158 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 */
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <time.h>
+#include <sys/time.h>
+
+#ifdef OPENMP
+#include <omp.h>
+#endif
 
 #include "globals.h"
 #include "readSet.h"
 #include "splay.h"
 #include "tightString.h"
-#include "crc.h"
 #include "utility.h"
 #include "kmer.h"
 #include "kmerOccurenceTable.h"
+#include "recycleBin.h"
+
+
+#define HASH_BUCKETS_NB 16777216
+
+#ifdef OPENMP
+
+#define NB_PUSH 32
+#define BUFFER_SIZE 4096
+
+static StringBuffer **annotationBuffer = NULL;
+static StringBuffer **annotationBufferW = NULL;
+static int *nbPush = NULL;
+static boolean producing = 1;
+
+static void initAnnotationBuffers(void)
+{
+	int n;
+	int i;
+
+	n = omp_get_max_threads();
+	annotationBuffer = callocOrExit(n, StringBuffer*);
+	annotationBufferW = callocOrExit(n, StringBuffer*);
+	nbPush = callocOrExit(n, int);
+
+	for (i = 0; i < n; i++)
+	{
+		annotationBuffer[i] = newStringBuffer(BUFFER_SIZE);
+		annotationBufferW[i] = newStringBuffer(BUFFER_SIZE);
+	}
+}
+
+static void destroyAnnotationBuffers(void)
+{
+	int n;
+	int i;
+
+	n = omp_get_max_threads();
+
+	for (i = 0; i < n; i++)
+	{
+		destroyStringBuffer(annotationBuffer[i], 1);
+		destroyStringBuffer(annotationBufferW[i], 1);
+	}
+
+	free(annotationBuffer);
+	free(annotationBufferW);
+	free(nbPush);
+	annotationBuffer = NULL;
+	annotationBufferW = NULL;
+	nbPush = NULL;
+}
+
+static void pushBufferCommit(int thread)
+{
+	StringBuffer *tmp;
+	char *s;
+
+	s = annotationBufferW[thread]->str;
+	do
+	{
+		#pragma omp flush(s)
+	}
+	while (*s);
+	tmp = annotationBufferW[thread];
+	annotationBufferW[thread] = annotationBuffer[thread];
+	annotationBuffer[thread] = tmp;
+	tmp = annotationBufferW[thread];
+	#pragma omp flush(tmp)
+}
+
+static void pushBuffer(int thread)
+{
+	if (++nbPush[thread] == NB_PUSH)
+	{
+		nbPush[thread] = 0;
+		pushBufferCommit(thread);
+	}
+}
+
+static void writeBuffers(FILE *outFile, int nbThreads)
+{
+	int i;
+
+	for (i = 0; i < nbThreads; i++)
+	{
+		StringBuffer *b;
+		char *s;
+
+		b = annotationBufferW[i];
+		#pragma omp flush(b)
+		s = b->str;
+		#pragma omp flush(s)
+		if (*s)
+		{
+			velvetFprintf(outFile, "%s", annotationBufferW[i]->str);
+			resetStringBuffer(annotationBufferW[i]);
+		}
+	}
+}
+
+static void bufferWritter(FILE *outFile)
+{
+	int n;
+
+	n = omp_get_max_threads();
+	#pragma omp flush(producing)
+	while (producing)
+	{
+		writeBuffers(outFile, n);
+		#pragma omp flush(producing)
+	}
+	writeBuffers(outFile, n);
+}
+
+static void appendLine(char *line, int thread)
+{
+	appendStringBuffer(annotationBuffer[thread], line);
+}
+#else
+
+#define BUFFER_SIZE 1024
+
+StringBuffer *annotationBuffer = NULL;
+
+static void appendLine(char *line, int thread)
+{
+	appendStringBuffer(annotationBuffer, line);
+}
+#endif
 
 struct splayTable_st {
 	SplayTree **table;
+#ifdef OPENMP
+	omp_lock_t *tableLocks;
+#endif
 	KmerOccurenceTable *kmerOccurenceTable;
-	IDnum lastIndex;
 	int WORDLENGTH;
 	boolean double_strand;
 };
@@ -43,10 +180,17 @@ SplayTable *newSplayTable(int WORDLENGTH, boolean double_strand)
 {
 	SplayTable *splayTable = mallocOrExit(1, SplayTable);
 	splayTable->WORDLENGTH = WORDLENGTH;
-	splayTable->table = callocOrExit(CRC_HASH_BUCKETS, SplayTree *);
-	splayTable->lastIndex = 0;
+	splayTable->table = callocOrExit(HASH_BUCKETS_NB, SplayTree *);
 	splayTable->kmerOccurenceTable = NULL;
 	splayTable->double_strand = double_strand;
+#ifdef OPENMP
+	splayTable->tableLocks = mallocOrExit(HASH_BUCKETS_NB, omp_lock_t);
+	int i;
+	#pragma omp parallel for
+	for (i = 0; i < HASH_BUCKETS_NB; i++)
+		omp_init_lock(splayTable->tableLocks + i);
+	initSplayTreeMemory();
+#endif
 	return splayTable;
 }
 
@@ -62,9 +206,44 @@ void destroySplayTable(SplayTable * splayTable)
 	velvetLog("Splay table destroyed\n");
 }
 
-static int hash_kmer(Kmer * kmer)
+static KmerKey hash_kmer(Kmer * kmer)
 {
-	return crc32_v((char *) kmer, KMER_BYTE_SIZE);
+#if KMER_LONGLONGS
+	KmerKey key = kmer->longlongs[0];
+
+#if KMER_LONGLONGS > 1
+	key ^= kmer->longlongs[1];
+#endif
+#if KMER_LONGLONGS > 2
+	key ^= kmer->longlongs[2];
+#endif
+
+	key = (~key) + (key << 21);
+	key = key ^ (key >> 24);
+	key = (key + (key << 3)) + (key << 8);
+	key = key ^ (key >> 14);
+	key = (key + (key << 2)) + (key << 4);
+	key = key ^ (key >> 28);
+	key = key + (key << 31);
+
+	return key % HASH_BUCKETS_NB;
+#elif KMER_LONGS
+	KmerKey key = kmer->longs[0];
+
+	key += ~(key << 15);
+	key ^= (key >> 10);
+	key += (key << 3);
+	key ^= (key >> 6);
+	key += ~(key << 11);
+	key ^= (key >> 16);
+
+	return key % HASH_BUCKETS_NB;
+
+#elif KMER_INTS
+	return kmer->ints % HASH_BUCKETS_NB;
+#elif KMER_CHARS
+	return kmer->chars % HASH_BUCKETS_NB;
+#endif
 }
 
 static Coordinate getNearestHSPIndex(Coordinate position, IDnum * sequenceIDs, Coordinate sequenceLength) {
@@ -137,6 +316,26 @@ static KmerOccurence * getMostAppropriateHit(Coordinate readCoord, Coordinate re
 	return best;
 }
 
+static inline boolean
+doFindOrInsertOccurenceInSplayTree(Kmer * kmer, IDnum * seqID,
+				   Coordinate * position, SplayTable *table)
+{
+#ifdef OPENMP
+	const KmerKey kmerHash = hash_kmer(kmer);
+	boolean ret;
+
+	omp_set_lock(table->tableLocks + kmerHash);
+	ret =  findOrInsertOccurenceInSplayTree(kmer, seqID, position,
+						table->table + kmerHash);
+	omp_unset_lock(table->tableLocks + kmerHash);
+
+	return ret;
+#else
+	return findOrInsertOccurenceInSplayTree(kmer, seqID, position,
+						&table->table[hash_kmer(kmer)]);
+#endif
+}
+
 
 static boolean findOrInsertOccurenceInSplayTable(Kmer * kmer, IDnum * seqID,
 						 Coordinate * position,
@@ -161,9 +360,7 @@ static boolean findOrInsertOccurenceInSplayTable(Kmer * kmer, IDnum * seqID,
 	}
 	else if (coords && coords[readIndex]) 
 		// If in buffer zone:
-		return findOrInsertOccurenceInSplayTree(kmer, seqID, position,
-							&table->
-							table[hash_kmer(kmer)]);
+		return doFindOrInsertOccurenceInSplayTree(kmer, seqID, position, table);
 
 	// Look up first in reference sequence k-mers
 	if (table->kmerOccurenceTable 
@@ -184,14 +381,13 @@ static boolean findOrInsertOccurenceInSplayTable(Kmer * kmer, IDnum * seqID,
 	} 
 
 	// If not, go through the novel k-mers
-	return findOrInsertOccurenceInSplayTree(kmer, seqID, position,
-						&table->
-						table[hash_kmer(kmer)]);
+	return doFindOrInsertOccurenceInSplayTree(kmer, seqID, position, table);
 }
 
-static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightString * tString, SplayTable * table, FILE * file, boolean second_in_pair) 
+static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords,
+			     TightString * array, SplayTable * table,
+			     FILE * file, boolean second_in_pair, IDnum seqID) 
 {
-	IDnum currentIndex;
 	Coordinate readNucleotideIndex = 0;
 	Coordinate writeNucleotideIndex = 0;
 	Kmer word;
@@ -205,17 +401,28 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 	Coordinate finish = 0;
 	IDnum referenceSequenceID = 0;
 	Nucleotide nucleotide;
+	char lineBuffer[MAXLINE];
+	TightString * tString = getTightStringInArray(array, seqID - 1); 
+	int thread = 0;
 
 	clearKmer(&word);
 	clearKmer(&antiWord);
 
-	table->lastIndex++;
+#ifdef OPENMP
+	thread = omp_get_thread_num();
+#endif
 
-	currentIndex = table->lastIndex;
-	velvetFprintf(file, "ROADMAP %li\n", (long) currentIndex);
+	sprintf(lineBuffer, "ROADMAP %li\n", (long)seqID);
+	appendLine(lineBuffer, thread);
 
 	// Neglect any string shorter than WORDLENGTH :
 	if (getLength(tString) < table->WORDLENGTH) {
+#ifdef OPENMP
+		pushBuffer(thread);
+#else
+		velvetFprintf(file, "%s", annotationBuffer->str);
+		resetStringBuffer(annotationBuffer);
+#endif
 		return;
 	}
 
@@ -242,7 +449,7 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 		reversePushNucleotide(&antiWord, 3 - nucleotide);
 #endif
 
-		sequenceID = currentIndex;
+		sequenceID = seqID;
 		coord = writeNucleotideIndex;
 
 		if (table->double_strand) {
@@ -302,9 +509,10 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 		if (!found) {
 			writeNucleotideIndex++;
 			if (!annotationClosed) {
-				velvetFprintf(file, "%ld\t%lld\t%lld\t%lld\n",
+				sprintf(lineBuffer, "%ld\t%lld\t%lld\t%lld\n",
 					(long) referenceSequenceID, (long long) position,
 					(long long) start, (long long) finish);
+				appendLine(lineBuffer, thread);
 			}
 			annotationClosed = true;
 		}
@@ -337,9 +545,10 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 			}
 			// Previous non corresponding annotation
 			else {
-				velvetFprintf(file, "%ld\t%lld\t%lld\t%lld\n",
+				sprintf(lineBuffer, "%ld\t%lld\t%lld\t%lld\n",
 					(long) referenceSequenceID, (long long) position,
 					(long long) start, (long long) finish);
+				appendLine(lineBuffer, thread);
 
 				referenceSequenceID = sequenceID;
 				position = writeNucleotideIndex;
@@ -356,15 +565,22 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 	}
 
 	if (!annotationClosed) {
-		velvetFprintf(file, "%ld\t%lld\t%lld\t%lld\n",
+		sprintf(lineBuffer, "%ld\t%lld\t%lld\t%lld\n",
 			(long) referenceSequenceID, (long long) position,
 			(long long) start, (long long) finish);
+		appendLine(lineBuffer, thread);
 	}
+#ifdef OPENMP
+	pushBuffer(thread);
+#else
+	velvetFprintf(file, "%s", annotationBuffer->str);
+	resetStringBuffer(annotationBuffer);
+#endif
 
 	return;
 }
 
-static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean second_in_pair, SplayTable * table, IDnum * sequenceIDs, Coordinate * coords) {
+static void computeClearHSPs(TightString * array, FILE * seqFile, boolean second_in_pair, SplayTable * table, IDnum ** sequenceIDs, Coordinate ** coords, IDnum * seqID) {
 	Coordinate readNucleotideIndex = 0;
 	Kmer word;
 	Kmer antiWord;
@@ -372,6 +588,7 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 	Nucleotide nucleotide;
 	KmerOccurence * hit;
 	char line[MAXLINE];
+	char c;
 	
 	Coordinate mapCount = 0;
 	Coordinate maxCount = 10;	
@@ -380,26 +597,54 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 	long long_var;
 	long long longlong_var;
 	int penalty;
+	TightString * tString;
+	Coordinate length;
 
 	clearKmer(&polyA);
 
-	// Parse file for mapping info
-	while (seqFile && fgets(line, MAXLINE, seqFile)) {
-		if (line[0] == '>')
-			break;
-	
-		if (line[0] == 'M') {
-			sscanf(line,"M\t%li\t%lli\n", &long_var, &longlong_var);
-			mapReferenceIDs[mapCount] = (IDnum) long_var;
-			mapCoords[mapCount] = (Coordinate) longlong_var;
+#ifdef OPENMP
+	#pragma omp critical
+	{
+#endif
+		// Get read ID:
+		if (!fgets(line, MAXLINE, seqFile)) {
+			puts("Incomplete Sequences file (computeHSPScores)");
+#ifdef DEBUG
+			abort();
+#endif 
+			exit(1);
+		}
+		sscanf(line, "%*s\t%li\t", &long_var);
+		*seqID = (IDnum) long_var;
+		if (*seqID == 0)
+			abort();
+		tString = getTightStringInArray(array, *seqID - 1);
+		length = getLength(tString);
+		*sequenceIDs = callocOrExit(length, IDnum);
+		*coords = callocOrExit(length, Coordinate);
 
-			if (++mapCount == maxCount) {
-				maxCount *= 2;
-				mapReferenceIDs = reallocOrExit(mapReferenceIDs, maxCount, IDnum);
-				mapCoords = reallocOrExit(mapCoords, maxCount, Coordinate);
+		// Parse file for mapping info
+		while (seqFile && (c = getc(seqFile)) != EOF) {
+			if (c == '>')
+				break;
+
+			fgets(line, MAXLINE, seqFile);
+
+			if (c == 'M') {
+				sscanf(line,"\t%li\t%lli\n", &long_var, &longlong_var);
+				mapReferenceIDs[mapCount] = (IDnum) long_var;
+				mapCoords[mapCount] = (Coordinate) longlong_var;
+
+				if (++mapCount == maxCount) {
+					maxCount *= 2;
+					mapReferenceIDs = reallocOrExit(mapReferenceIDs, maxCount, IDnum);
+					mapCoords = reallocOrExit(mapCoords, maxCount, Coordinate);
+				}
 			}
 		}
+#ifdef OPENMP
 	}
+#endif
 
 	// First pass for unambiguous hits
 	// Fill in the initial word : 
@@ -444,44 +689,44 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 				hit = findKmerInKmerOccurenceTable(&word, table->kmerOccurenceTable); 
 
 				if (hit && (hit = getMostAppropriateHit(readNucleotideIndex, getLength(tString), true, hit, mapCount, mapReferenceIDs, mapCoords, table->WORDLENGTH))) 
-					sequenceIDs[readNucleotideIndex] = getKmerOccurenceNodeID(hit);
+					(*sequenceIDs)[readNucleotideIndex] = getKmerOccurenceNodeID(hit);
 			} else {
 				hit = findKmerInKmerOccurenceTable(&antiWord, table->kmerOccurenceTable); 
 
 				if (hit && (hit = getMostAppropriateHit(readNucleotideIndex, getLength(tString), false, hit, mapCount, mapReferenceIDs, mapCoords, table->WORDLENGTH))) 
-					sequenceIDs[readNucleotideIndex] = -getKmerOccurenceNodeID(hit);
+					(*sequenceIDs)[readNucleotideIndex] = -getKmerOccurenceNodeID(hit);
 			}
 		} else {
 			if (!second_in_pair) {
 				hit = findKmerInKmerOccurenceTable(&word, table->kmerOccurenceTable); 
 
 				if (hit && (hit = getMostAppropriateHit(readNucleotideIndex, getLength(tString), true, hit, mapCount, mapReferenceIDs, mapCoords, table->WORDLENGTH))) 
-					sequenceIDs[readNucleotideIndex] = getKmerOccurenceNodeID(hit);
+					(*sequenceIDs)[readNucleotideIndex] = getKmerOccurenceNodeID(hit);
 			} else {
 				hit = findKmerInKmerOccurenceTable(&antiWord, table->kmerOccurenceTable); 
 
 				if (hit && (hit = getMostAppropriateHit(readNucleotideIndex, getLength(tString), false, hit, mapCount, mapReferenceIDs, mapCoords, table->WORDLENGTH))) 
-					sequenceIDs[readNucleotideIndex] = -getKmerOccurenceNodeID(hit);
+					(*sequenceIDs)[readNucleotideIndex] = -getKmerOccurenceNodeID(hit);
 			}
 		}
 
-		if (sequenceIDs[readNucleotideIndex]) {
-			if (sequenceIDs[readNucleotideIndex] > 0) 
-				coords[readNucleotideIndex] = getKmerOccurencePosition(hit) - readNucleotideIndex;
+		if ((*sequenceIDs)[readNucleotideIndex]) {
+			if ((*sequenceIDs)[readNucleotideIndex] > 0) 
+				(*coords)[readNucleotideIndex] = getKmerOccurencePosition(hit) - readNucleotideIndex;
 			else
-				coords[readNucleotideIndex] = getKmerOccurencePosition(hit) + readNucleotideIndex - getLength(tString) + 1;
+				(*coords)[readNucleotideIndex] = getKmerOccurencePosition(hit) + readNucleotideIndex - getLength(tString) + 1;
 		}
 	
 		// Barrier to flip-flopping
-		if (sequenceIDs[readNucleotideIndex - 1] != 0
-		    && (sequenceIDs[readNucleotideIndex] != sequenceIDs[readNucleotideIndex - 1]
-			|| coords[readNucleotideIndex] != coords[readNucleotideIndex - 1])) {
+		if ((*sequenceIDs)[readNucleotideIndex - 1] != 0
+		    && ((*sequenceIDs)[readNucleotideIndex] != (*sequenceIDs)[readNucleotideIndex - 1]
+			|| (*coords)[readNucleotideIndex] != (*coords)[readNucleotideIndex - 1])) {
 			// Break in continuity... skip k positions 
-			sequenceIDs[readNucleotideIndex] = 0;
-			coords[readNucleotideIndex] = -1;
+			(*sequenceIDs)[readNucleotideIndex] = 0;
+			(*coords)[readNucleotideIndex] = -1;
 			readNucleotideIndex++;
 
-			for (penalty = 0; readNucleotideIndex < getLength(tString) && (penalty < table->WORDLENGTH  - 1 || compareKmers(&word, &polyA) == 0 || compareKmers(&antiWord, &polyA) == 0); penalty++) {
+			for (penalty = 0; penalty < table->WORDLENGTH  - 1 && readNucleotideIndex < getLength(tString); penalty++) {
 				nucleotide = getNucleotide(readNucleotideIndex, tString);
 				pushNucleotide(&word, nucleotide);
 
@@ -490,8 +735,8 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 #else
 				reversePushNucleotide(&antiWord, 3 - nucleotide);
 #endif
-				sequenceIDs[readNucleotideIndex] = 0;
-				coords[readNucleotideIndex] = -1;
+				(*sequenceIDs)[readNucleotideIndex] = 0;
+				(*coords)[readNucleotideIndex] = -1;
 				readNucleotideIndex++;
 			}
 		} else
@@ -503,23 +748,21 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 	free(mapCoords);
 }
 
-void inputSequenceIntoSplayTable(TightString * tString,
-				 SplayTable * table, FILE * file, FILE * seqFile, boolean second_in_pair)
+void inputSequenceIntoSplayTable(TightString * array,
+				 SplayTable * table,
+				 FILE * file, FILE * seqFile,
+				 boolean second_in_pair,
+				 IDnum seqID)
 {
-	Coordinate length = getLength(tString);
 	IDnum * sequenceIDs = NULL;
 	Coordinate * coords = NULL;
 
 	// If appropriate, get the HSPs on reference sequences
-	if (table->kmerOccurenceTable) {
-		length = getLength(tString);
-		sequenceIDs = callocOrExit(length, IDnum);
-		coords = callocOrExit(length, Coordinate);
-		computeClearHSPs(tString, seqFile, second_in_pair, table, sequenceIDs, coords);
-	}
-	
+	if (table->kmerOccurenceTable) 
+		computeClearHSPs(array, seqFile, second_in_pair, table, &sequenceIDs, &coords, &seqID);
+
 	// Go through read, eventually with annotations
-	printAnnotations(sequenceIDs, coords, tString, table, file, second_in_pair);
+	printAnnotations(sequenceIDs, coords, array, table, file, second_in_pair, seqID);
 
 	// Clean up
 	if (sequenceIDs) {
@@ -529,7 +772,7 @@ void inputSequenceIntoSplayTable(TightString * tString,
 }
 
 void inputReferenceIntoSplayTable(TightString * tString,
-				 SplayTable * table, FILE * file)
+				 SplayTable * table, FILE * file, IDnum seqID)
 {
 	IDnum currentIndex;
 	Coordinate readNucleotideIndex = 0;
@@ -537,14 +780,20 @@ void inputReferenceIntoSplayTable(TightString * tString,
 	Kmer word;
 	Kmer antiWord;
 	Nucleotide nucleotide;
+#ifdef OPENMP
+	char lineBuffer[MAXLINE];
+#endif
 
 	clearKmer(&word);
 	clearKmer(&antiWord);
 
-	table->lastIndex++;
-
-	currentIndex = table->lastIndex;
-	velvetFprintf(file, "ROADMAP %li\n", (long) currentIndex);
+	currentIndex = seqID;
+#ifdef OPENMP
+	sprintf(lineBuffer, "ROADMAP %li\n", (long)currentIndex);
+	appendLine(lineBuffer, omp_get_thread_num());
+#else
+	velvetFprintf(file, "ROADMAP %li\n", (long)currentIndex);
+#endif
 
 	// Neglect any string shorter than WORDLENGTH :
 	if (getLength(tString) < table->WORDLENGTH) {
@@ -626,7 +875,8 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 	IDnum kmerCount;
 	IDnum referenceSequenceCount = 0;
 	char line[MAXLINE];
-	boolean second_in_pair = false;
+	struct timeval start, end, diff;
+	char c;
 
 	if (outfile == NULL)
 		exitErrorf(EXIT_FAILURE, true, "Couldn't write to file %s", filename);
@@ -642,7 +892,23 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 	if (reads->tSequences == NULL)
 		convertSequences(reads);
 
-	if (referenceSequenceCount && (kmerCount = countReferenceKmers(reads, table->WORDLENGTH))> 0) {
+	gettimeofday(&start, NULL);
+	array = reads->tSequences;
+
+#ifdef OPENMP
+	if (omp_get_max_threads() == 1)
+	{
+		omp_set_num_threads(2);
+		omp_set_nested(0);
+	}
+	else
+		omp_set_nested(1);
+	initAnnotationBuffers();
+#else
+	annotationBuffer = newStringBuffer(BUFFER_SIZE);
+#endif
+
+	if (referenceSequenceCount && (kmerCount = countReferenceKmers(reads, table->WORDLENGTH)) > 0) {
 		table->kmerOccurenceTable = newKmerOccurenceTable(24 , table->WORDLENGTH);
 		allocateKmerOccurences(kmerCount, table->kmerOccurenceTable);
 		seqFile = fopen(seqFilename, "r");
@@ -652,60 +918,111 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 		else
 			velvetLog("Reading mapping info from file %s\n", seqFilename);
 
-		for (index = 0; index < referenceSequenceCount + 1; index++) 
-			while (fgets(line, MAXLINE, seqFile))
+		// Skip through reference headers quickly
+		for (index = 0; index < referenceSequenceCount; index++)
+			while (fgets(line, MAXLINE, seqFile)) 
 				if (line[0] == '>')
 					break;
+
+		// Edge to the first sequence header
+		while ((c = getc(seqFile))) {
+			if (c == '>')
+				break;
+			fgets(line, MAXLINE, seqFile);
+		}
+
+#ifdef OPENMP
+		producing = 1;
+		#pragma omp parallel sections
+		{
+			#pragma omp section
+			{
+				bufferWritter(outfile);
+			}
+			#pragma omp section
+			{
+				#pragma omp parallel for
+#endif
+				for (index = 0; index < referenceSequenceCount; index++)
+					inputReferenceIntoSplayTable(getTightStringInArray(array, index),
+								     table, outfile, index + 1);
+
+#ifdef OPENMP
+				for (index = omp_get_max_threads() - 1; index >= 0; index--)
+					pushBufferCommit(index);
+				producing = 0;
+				#pragma omp flush(producing)
+			}
+		}
+#endif
+
+		sortKmerOccurenceTable(table->kmerOccurenceTable);
 	}
 
 	velvetLog("Inputting sequences...\n");
-	array = reads->tSequences;
-	for (index = 0; index < sequenceCount; index++) {
-		// Prorgess report on screen
-		if (index % 100000 == 0) {
-			velvetLog("Inputting sequence %li / %li\n", (long) index,
-			       (long) sequenceCount);
-			fflush(stdout);
-		}
 
-		// Before starting non-reference sequences, sort kmerOccurenceTable
-		if (index > 0 
-		    && reads->categories[index - 1] == REFERENCE
-		    && reads->categories[index] != REFERENCE)
-			sortKmerOccurenceTable(table->kmerOccurenceTable);
-		// Test to make sure that all the reference reads are before all the other reads
-		else if (index > 0
-		    && reads->categories[index - 1] != REFERENCE
-		    && reads->categories[index] == REFERENCE) {
-			velvetLog("Reference sequence placed after a non-reference read!\n");
-			velvetLog(">> Please re-order the filenames in your command line so as to have the reference sequence files before all the others\n");
+#ifdef OPENMP
+	producing = 1;
+	#pragma omp parallel sections
+	{
+		#pragma omp section
+		{
+			bufferWritter(outfile);
+		}
+		#pragma omp section
+		{
+			#pragma omp parallel for
+#endif
+			for (index = referenceSequenceCount; index < sequenceCount; index++)
+			{
+				boolean second_in_pair;
+
+				// Progress report on screen
+				if (index % 1000000 == 0) {
+					velvetLog("Inputting sequence %li / %li\n",
+						  (long)index, (long)sequenceCount);
+					fflush(stdout);
+				}
+
+				// Test to make sure that all the reference reads are before all the other reads
+				if (reads->categories[index] == REFERENCE) {
+					velvetLog("Reference sequence placed after a non-reference read!\n");
+					velvetLog(">> Please re-order the filenames in your command line so as "
+						  "to have the reference sequence files before all the others\n");
 #ifdef DEBUG 
-			abort();
+					abort();
 #endif 
-			exit(0);
+					exit(0);
+				}
+				second_in_pair = reads->categories[index] % 2 && isSecondInPair(reads, index);
+
+				// Hashing the reads
+				inputSequenceIntoSplayTable(array, table,
+							    outfile, seqFile,
+							    second_in_pair, index + 1);
+			}
+#ifdef OPENMP
+			for (index = omp_get_max_threads() - 1; index >= 0; index--)
+				pushBufferCommit(index);
+			producing = 0;
+			#pragma omp flush(producing)
 		}
-
-		// Hashing the reads
-		if (reads->categories[index] == REFERENCE)
-			// Reference reads
-			inputReferenceIntoSplayTable(getTightStringInArray(array, index), table, outfile);
-		else
-			// Normal reads
-			inputSequenceIntoSplayTable(getTightStringInArray(array, index), table, outfile, seqFile, second_in_pair);
-
-		if (reads->categories[index] % 2) 
-			second_in_pair = (second_in_pair? false : true);
-		else 
-			second_in_pair = false;
-
 	}
+	destroyAnnotationBuffers();
+#else
+	destroyStringBuffer(annotationBuffer, 1);
+#endif
+
+	gettimeofday(&end, NULL);
+	timersub(&end, &start, &diff);
+	velvetLog(" === Sequences loaded in %ld.%06ld s\n", diff.tv_sec, diff.tv_usec);
 
 	fclose(outfile);
 	if (seqFile)
 		fclose(seqFile);
 
-	free(reads->tSequences);
-	reads->tSequences = NULL;
-	destroyReadSet(reads);
+	//free(reads->tSequences);
+	//reads->tSequences = NULL;
+	//destroyReadSet(reads);
 	velvetLog("Done inputting sequences\n");
 }
